#!/usr/bin/python -u
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: BDTSconverter.py,v 1.2 2008-11-06 11:26:18 rguillie Exp $


from xml.sax import saxutils
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces
import string


def normalize_whitespace(text):
    "Remove redundant whitespace from a string"
    return ' '.join(text.split())


class BDTSProfileParser(saxutils.DefaultHandler):
    "XML Parsing class to generate the internal association between a node name and the string corresponding to its bandwith profile, for BDTS usage"

    #A hashtable to maintain the association node <-> profile
    nodeTable = {}

    #The mandatory string to initiate the connection 
    connectstring = "C1 := connect localhost\n"
    #The string used to construct the profile
    outputstring = connectstring
    
    keynb        = 0
    nodename     = ""
    
    #Special treatment for the first date/rate encountered
    firstdate    = True
    firstrate    = True
    
    #Boolean to indicate that we are in the correct context
    inscope = False
    tagtype = 0
    
    def startElement(self, name, attrs):
        
        if self.inscope: 
            if name == 'key':        
                self.tagtype = 1
            elif name == 'date':
                self.tagtype = 2
            elif name == 'rate':
                self.tagtype = 3
            elif name == 'name':
                self.tagtype = 4
            elif name == 'node':
                #New node, reinitialisation of all vars 
                self.tagtype = 0 
                self.outputstring = self.connectstring
                self.firstrate = True
                self.firstdate = True
                self.nodename = ""
            else:
                #We don't care about the other tags
                self.tagtype = 0
        else:              
            if name == 'profile':
                #we are in the scope of a 
                self.inscope = True           
           
           

    def characters(self,ch):
        if self.inscope:
            if (self.tagtype == 1):
                self.keynb = normalize_whitespace(ch)
            elif (self.tagtype == 2):
                self.outputstring = self.outputstring + "[" + normalize_whitespace(ch) + "] "
            elif (self.tagtype == 3):
                self.outputstring += normalize_whitespace(ch) 
            elif (self.tagtype == 4):
                self.nodename = normalize_whitespace(ch)
            #else:
            #    print "Oups, invalid tagtype"
        
        self.tagtype=0

    def endElement(self,name):

        if self.inscope:

            if name == 'date':
                if self.firstdate:
                    self.firstdate = False
                    self.outputstring += "L0 := {C1} limit "
                else:
                    self.outputstring += "modify limit L0 "
            elif name == 'rate':
                if self.firstrate:
                    self.firstrate = False
                    self.outputstring = self.outputstring + " token " + self.keynb
            elif name == 'step':
                self.outputstring += "\n"
            elif name == 'node':
                #Adding the node to the hash-table
                self.nodeTable[self.nodename] = self.outputstring
            elif name == 'profile':
                #End of the profile, getting out of context 
                self.inscope=False


if __name__ == '__main__':

    from sys import argv 

    if len(argv) <= 1 :
        filename='scenario.xml'
    else:
        filename=argv[1]
    
    	
    file = open(filename, 'r')


    # Create a parser
    parser = make_parser()

    # Tell the parser we are not interested in XML namespaces
    parser.setFeature(feature_namespaces, 0)

    # Create the handler
    dh = BDTSProfileParser()

    # Tell the parser to use our handler
    parser.setContentHandler(dh)

    # Parse the input
    parser.parse(file)

    for node, profile in dh.nodeTable.iteritems():
        nodefilename = "profile_" + node + ".floc"
        nodefile = open(nodefilename, 'w');
        nodefile.write(profile)
        nodefile.close()

    file.close()



