#! /usr/bin/python -u
#A module containing the main function to execute a given scenario
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXE.py,v 1.9 2008-11-06 11:26:18 rguillie Exp $

import os
import sys
import string
import paramiko
import subprocess
import datetime 
import shutil
import popen2


from xml.sax import saxutils
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces

import NXELogging
#logging = NXELogging.NXELogging()


import NXETypes
import NXEUtils 
import NXEParser
import NXEThread
import NXEStats
import NXEChart
import NXELogHandling


from NXEUtils import getTime, split_hostname
from NXELogging import State, logging
from NXEThread import NXEThreadTimer, NXEThreadCommand, NXEThreadConnecting, NXEThreadScript
from paramiko import SSHClient

#TODO: preallocation of all tab whose size are known, it is more efficient
#TODO: find a proper Python way to do the "ls pattern" instead of the popen2 subprocesses as it isn't portable 
#TODO: consistency check every time we try to acces a given key in a dict structure
#TODO: A real logging module because there is no consistency about the way it is done and in the other modules




tmpfolder = ""
archivefolder = ""

# Create the handlers
config = NXEParser.ConfigParser()
scenario = NXEParser.ScenarioParser()
topology = NXEParser.TopologyParser()
profile  = NXEParser.BDTSProfileParser()

pid = str(os.getpid())
spoofed = False

                                    
    
#The global idea around here is to use one or several scripts that
#will do the actual job on the remote site and provide/fill the information
#that are needed by the following steps


###
def parsing_config(filename):
    """A method handling the parsing of the config file
    It is clearly not optimal as we are performing at least 3 passes on the XML file
    But it is easier to have a parser for each big section of the file
    """

    global pid
    global spoofed

    #file = open(filename, 'r')

    # Create a parser
    parser = make_parser()

    # Tell the parser we are not interested in XML namespaces
    parser.setFeature(feature_namespaces, 0)

    # Tell the parser to use our handler
    parser.setContentHandler(config)
    # Parse the input
    parser.parse(filename)

    parser.setContentHandler(scenario)
    parser.parse(filename)

    parser.setContentHandler(topology)
    parser.parse(filename)

    #Only doing it if we are in a BDTS environment
    if config.BDTS:
        parser.setContentHandler(profile)
        parser.parse(filename)

    #TODO: need to add something to reconstruct a reservation if the tmp files
    #got erased 
    #Used only for test/reusing of already made reservation     
    if config.spoof != -1:
        pid = str(config.spoof)
        spoofed = True
	logging.printOut("using a spoofed pid " + pid) 

    #print pid
    #print config.spoof


    #file.close

    #Parsing of the options is done
    
###
def handling_reservation(siteList, frontalList, nodeList, date):
    """A function that is supposed to handle the reservation part

       The script handling the  \"low level\" reservation SHOULD
       return at least a column of IP/hostnames (if there are more,
       only the second one is interpreted as an alternate IP

       If less than expected nodes are reserved, we check that
       enough nodes are still there to continue the process (according
       to user-input in the scenario entry file)
    """
    
    stdoutList = []
    sleep = 0

    import datetime 
    from datetime import timedelta,datetime

    
    if date != "":

        version = string.split(string.split(sys.version)[0], ".")

        #TODO Remove this when we switch to a more recent version of the python interpreter
        if map(int,version) >= [ 2, 5 ]:
            futuredate = datetime.strptime(date, "%Y-%m-%d %H:%M:%S")
        #print sleep
        else:
            logging.printOut("Warning: Date calculation for reservation in advance done manually with this python version " + str(version), NXELogging.LOG_WARN)
            dateandtime=date.split()
            splitdate = dateandtime[0].split('-')
            splittime = dateandtime[1].split(':')
            futuredate = datetime(int(splitdate[0]),int(splitdate[1]),int(splitdate[2]),int(splittime[0]),int(splittime[1]), int(splittime[2]))

        deltatime = futuredate - datetime.now()
        sleep = deltatime.days * 24 * 60 * 60 + deltatime.seconds 
        

    index =0
    for site, sitedescr in siteList.iteritems():

        if site not in frontalList:
            logging.printErr("no frontal node for site " + site + ". Please check topology description in scenario")
            cleaning(siteList, frontalList)
            sys.exit()

        #print frontalList[site].frontalname

        #ssh'ing to the frontal corresponding to the site and
        #executing the script 
        client = SSHClient()
        client.load_system_host_keys()
        #Set the policy to accept automatically new keys
        #Especially useful if we need to connect to a changing pool of nodes like in G5K
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        user, host = split_hostname(frontalList[site].frontalname)
        client.connect(host, username=user)

        #Keeping the connexion 
        sitedescr.connexionssh = client
        
        
        #The expected behaviour of this script is to return
        #a list containing the list of the nodes
        #one per line, a second column can be used to provide
        
        command = frontalList[site].resatool + " --pid " + pid + " --cluster " + site + " --nodes " + str(sitedescr.number)       
	if spoofed:
            command = command + " --reservation-number " + pid
	if date != "":
            command = command + " --date \"" + date + "\" --sleep " + str(sleep) 
            
        #Finally adding the user's parameters 
        command = command + " " +  frontalList[site].resaparam


	logging.printOut(command)
        stdin, stdout, stderr = client.exec_command(command)
        #Saving the stdout, to read the reserved nodes later on
        #after having started all the stdout
        #Might be cleaner to do that with a new Thread class as the output
        #is different from what we expect from other scripts
        stdoutList.insert(index, [sitedescr.siteabstractname, (sitedescr.number - sitedescr.maxmissing), stdout])
        index += 1
        #print stdout.read()
        #print stderr.read()

    for i in xrange(0, len(stdoutList)):
        
        #reading and parsing stdout to get the list of nodes
        reservednodes = 0
        nodename = ""
        stdout = stdoutList[i][2]
        site = stdoutList[i][0]
        sitenumber = stdoutList[i][1]
        
        for line in stdout.readlines():
            #/!\ empty lines
            #print line
            
            ip=""
            alternateip=""
            reservednodes += 1
            nodename = site + str(reservednodes)
            splittedline = line.split()
            #According to split spec, we have either a empty list or the list containing our stuff
	    #print splittedline            
           
            if len(splittedline) <= 0:
                #Going back, this was an empty line
		reservednodes -= 1
		continue
            elif len(splittedline) == 1:
                ip = splittedline[0]
                alternateip = ip
	    elif len(splittedline) >= 2:                
               ip = splittedline[0] 
               alternateip = splittedline[1]

            #If there is more than 2 columns, we don't known yet what we need to do with it	        
            nodeList[nodename] = (ip,alternateip)
               
        #We didn't get the right/expected number of nodes for this site
        #Aborting everything is the most secure thing to do
        #print reservednodes,sitedescr.number 
	
	if sitenumber  > reservednodes :
            logging.printErr("wrong number of nodes reserved for site " + site + " " + str(reservednodes) + " instead of the needed " + str(sitedescr.number)) 
            cleaning(siteList, frontalList)
            sys.exit()
        
        #Updating the site number properly to avoid miscalculation at the following stage
        sitedescr.maxmissing = sitedescr.maxmissing - sitedescr.number + reservednodes
        sitedescr.number = reservednodes

    #Reservation is done, we just need to do the instanciation of the addresses in the nodes' BOM
        
###    
def handling_deployment(siteList, frontalList, nodeList):
    """
    A function that is supposed to handle the deployment/configuration part

    It is assumed that the \"low level\" script handling the deployment
    return OK! or KO! depending how well it worked.

    As an option, it can also sent back after that the name of the nodes
    for which the reservation failed.
    
    """
    #Perhaps it would be more convient de do the reservation/deployment part simultaneously
    
    if spoofed:
	return

    threadlist=[]

    #Here, it is better if we fork our remote ssh command as the deploy phase
    #is rather long usually
    for site, sitedescr in siteList.iteritems():

        
        commandstring = frontalList[site].deploytool + " --pid " + pid + " --cluster " + site + " " + frontalList[site].deployparam 
        
        logging.printOut(commandstring)
        current = NXEThreadCommand(commandstring, site, sitedescr.connexionssh)
        current.start()
        threadlist.append(current)

        
    #Joining all the threads again
    for thread in threadlist:
        thread.join()
        if not thread.isOK:
            #Usually, it might mean that we have nodes that have failed their deployment 
            print repr(thread.output)
            if len(thread.output) > 0 :               
                
                number_failed = len(thread.output)
                if ( number_failed > sitedescr.maxmissing):
                    logging.printErr("too many nodes missing on " + thread.location + " after deployment " + str(number_failed) + " instead of max " + str(sitedescr.maxmissing) +  ", aborting")
                    #TODO: need to call the cleaning stuff
                    sys.exit()
                    
                
                reconstructed_tab = []
                

                for i in xrange(1, sitedescr.number):
                    nodename= sitedescr.siteabstractname + str(i)
                    first, second = nodeList[nodename]
                                        #getting only the nodes that are not marked as failed
                    if first not in thread.output:
                        reconstructed_tab.append(nodeList[nodename])
                        
                for i in xrange(1, sitedescr.number - number_failed): 
                    nodename = sitedescr.siteabstractname + str(i) 
                    nodeList[nodename] = reconstructed_tab[i-1]

                for i in xrange(sitedescr.number - number_failed + 1, sitedescr.number): 
                    nodename = sitedescr.siteabstractname + str(i)
                    nodeList.pop(nodename)

            else:
                
                logging.printErr("problem with the deployment on site " + thread.location + ", aborting")
                cleaning(siteList, frontalList)
                sys.exit()
            
    #Deployment is done

###
def configure_gnet(siteList, frontalList, timeout):
    

    for site, sitedescr in siteList.iteritems():

    #Don't forget to manage the gnet flags !
        if sitedescr.gnet:
            #ssh'ing to the gnet corresponding to the site and
            #executing the script 
            client = SSHClient()
            client.load_system_host_keys()
            #Set the policy to accept automatically new keys
            #Especially useful if we need to connect to a changing pool of nodes like in G5K
            client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            user, host = split_hostname(sitedescr.gnetbox.gnetname)
            client.connect(host, username=user)

            command = sitedescr.gnetbox.gnettool + " " +  sitedescr.gnetbox.gnetparam + " --timeout " + str(timeout)         
	
            logging.printOut(command)
            stdin, stdout, stderr = client.exec_command(command)


            #TODO: Perhaps we should something with the output 
            #but we don't really need to do anything here


    
###
def ip_instanciation(nodeList, nodeIPAssociation):
    """A function to put the IP/real node names into the Nodes BOM
       It can be done as soon as we have all our nodes have been affected
       after the reservation/deployment part

       nodeList is the BOM created with the scenario
       nodeIPAssociation should be the association abstractNodeName <-> (contactIP, interfaceIP)

       the {contact,interface}IP could well be hostnames/DNS name rather than IP, the only thing is that we
       need the first one to do the ssh connexion, the second to transmit packets via the right interface

       The problem is that here, we are only considering the nodes that are appearing as an <id>
       (that's the way)  

       """

    for nodename, nodedescr in nodeList.iteritems():

        if nodename not in nodeIPAssociation:
            logging.printErr("Nothing corresponding to node " + nodename + ". Please check reservation/scenario")
            #Here we should fail directly, because a node that is appearing
            #in the scenario hasn't a corresponding "real" node 
            continue

        if len(nodeIPAssociation[nodename]) <= 0:
            logging.printErr("IP instanciation failed for node " + nodename)

        elif len(nodeIPAssociation[nodename]) == 1:
            nodedescr.interfaceIP = nodeIPAssociation[nodename][0]
	    nodedescr.contactIP = nodeIPAssociation[nodename][0]

        elif len(nodeIPAssociation[nodename]) >= 2:
            nodedescr.contactIP = nodeIPAssociation[nodename][0]
            nodedescr.interfaceIP = nodeIPAssociation[nodename][1]
	
	#print len((nodeIPAssociation[nodename]))
	#print nodename + "/" + nodedescr.contactIP + "/" + nodedescr.interfaceIP


    #Instanciation of the IP is done

###
def linking_profile_steps(nodeList, profileStepList):
    """A function to perform the association between the nodeList and the profile List
    """

    if len(profileStepList) <= 0:
        return

    for nodename, nodedescr in nodeList.iteritems():
        nodedescr.profileStepList = profileStepList[nodename]

    #Linking of the profile

###
def pairing_nodes(nodeList, user):
    """A function that is responsible for pairing the servers with the clients

       For the moment, we try to do something easy just by a double loop
       But it might be a good idea to check the validity of the pairing with
       respect to the topology description

       It supposes also (with the scenario description file) that a node's role
       is persistent throughout a scenario
    """
    import time

    from socket import gethostname;
    hostname = gethostname()
    login = os.getlogin()

    #Browsing through the nodes list to find the server nodes
    #serverList = []
    serverNumber = 0
    clientNumber = 0
    mpiNumber = 0

    threadList = []

    for node, nodedescr in nodeList.iteritems():

#       print node

        if nodedescr.nodeType == NXETypes.SERVER_NODE:
            #    serverList.append(node)
            serverNumber += 1
            #A server doesn't need a target
        elif nodedescr.nodeType == NXETypes.CLIENT_NODE:
            clientNumber += 1
            #print nodedescr.target
	
	    if nodedescr.target == "localhost":
		pass

	    if nodedescr.target in nodeList :
               nodedescr.target = nodeList[nodedescr.target].interfaceIP 
               #print nodedescr.target
       	    else:
                raise RuntimeError("No target " + nodedescr.target + " for node " + node + "/" + nodedescr.contactIP)  

	elif nodedescr.nodeType == NXETypes.MPI_NODE:
            mpiNumber += 1

            splittedcomma = NXEUtils.split_comma(nodedescr.target)
            filename = open("/tmp/mpi_node_file_" + pid, 'w')     
            for i in xrange(0, len(splittedcomma)):

                if splittedcomma[i] in nodeList:
                    filename.write(nodeList[splittedcomma[i]].interfaceIP + "\n")
                else:
                    raise RuntimeError("No target " + splittedcomma[i] + " for node " + node + "/" + nodedescr.contactIP)

            filename.close()

        current = NXEThreadConnecting(nodedescr.contactIP, node, user)
        current.start()
        time.sleep(0.05)
        threadList.append(current)

        #By the by, we can open the ssh connexions
        # client = SSHClient()
#         client.load_system_host_keys()
#         client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
#         client.connect(nodedescr.contactIP)
        #Keeping the connexion 
        #nodedescr.connexionssh = client

       
            
    #Joining the threads        
    for thread in threadList:
        thread.join()
        nodeList[thread.name].connexionssh = thread.connexion

        #Copying the file containing the nodes needed by the MPI process
        #Implicit convention that the script launching MPI processes will be looking for
        #a /tmp/mpi_node_file file
        if nodeList[thread.name].nodeType == NXETypes.MPI_NODE:
            commandstring = "scp -o CheckHostIP=no -o StrictHostKeyChecking=no " + login + '@' + hostname + ":" + "/tmp/mpi_node_file_" + pid + " " + user + '@' + nodeList[thread.name].contactIP + ":/tmp/mpi_node_file"   

            #print command_string
            stdin, stdout, stderr = thread.connexion.exec_command(commandstring)
            stdin.close()
            stderr.close()
            stdout.close()
        
    if serverNumber > clientNumber:
        logging.printOut("Warning, some servers weren't paired with clients", NXELogging.LOG_WARN)  
#        errorStream.write(getTime() + ": " + printCurrentState() + ": Server and Client number mismatch: " + serverNumber + " to " + clientNumber)  
        #Useless to continue if there is no server at all
#        raise RuntimeError

    #serverIterator = iter(serverList)    
    #Browsing through the nodes list to pair the client nodes
    #for node, nodedescr in nodeList.iteritems():
        
    #    if nodedescr.nodeType == NXETypes.CLIENT_NODE:
            #Pairing with the next available server
            #Looping again if none is available
    #        try:
    #            servername = serverIterator.next()
                
    #        except StopIteration:
    #            outputStream.write(getTime() + ": " +  printCurrentState() + ": Starting again the servers List")   
    #            serverIterator = iter(serverList)
    #            servername = serverIterator.next() 

    #        nodedescr.target = servername


###        
def handling_BDTS(nodeList, BDTSprogram):
    """ A method to handle the BDTS related stuff

        It assumes that the BDTS policies are handled on the nodes
        and that the profiles are already linked to the nodes
    """

    threadList = []
    for node, nodedescr in nodeList.iteritems():

        commandstring = BDTSprogram + nodedescr.profileStepList
        current = NXEThreadCommand(commandstring, nodedescr.contactIP, nodedescr.connexionssh)
        current.start()
        threadList.append(current)

    for thread in threadList:
        thread.join()
        if not thread.isOK:
            logging.printErr("problem with the BDTS configuration on node " + thread.location)
            #So what ? Should we abort when BDTS is failing ?
    
    #End of BDTS stuff

###
def preparing_schedule(nodeList, threadList, timeout, file, datelabel):
    """A function to prepare the schedule that will be run by the applications

       The basic idea is to create threads with timers that will execute
       the commands in the right order

        
    """
   
        
    #The ssh connexion should have been initiated in the pairing nodes method

    datetab = [] 
    #Creating the threads
    for node, nodedescr in nodeList.iteritems():

        sleepstring = ""
        sizestring = ""

        #The flag is set to use a law distribution from the two stored in the node
        #It is only relevant if the node is a client (eventually a MPI node, but that's not my
        #concern for the moment on)
        if (config.law and nodedescr.nodeType == NXETypes.CLIENT_NODE):
            index = 0


            #Launching the requested time the random to get the needed size/intersleep values
            while ( index < config.repeat):

                if (nodedescr.interdistrib is not None):
                    if (sleepstring == ""):
                        sleepstring = str(nodedescr.interdistrib.getRandom())
                    else:
                        sleepstring = sleepstring + ":" + str(nodedescr.interdistrib.getRandom())

                if (nodedescr.sizedistrib is not None):
                    if (sizestring == ""):
                        sizestring = str(nodedescr.sizedistrib.getRandom()) + "M"
                    else:
                        sizestring = sizestring + ":" + str(nodedescr.sizedistrib.getRandom()) + "M"


                index += 1
        
                



        if nodedescr.nodeType == NXETypes.SERVER_NODE:
        
           #Gathering all the labels for the later stage of diagramm generation
           for i in xrange(0, len(nodedescr.execStepList)):
            
               if not datelabel.has_key(nodedescr.execStepList[i].date):
                   if nodedescr.execStepList[i].label != "":
                       datelabel[nodedescr.execStepList[i].date] = nodedescr.execStepList[i].label
                   else:
                       datetab.append(nodedescr.execStepList[i].date)


        #Need to create the correct params -> add the pid/target in case of a client
        #Adding a "target" to the BOM might be necessary

        #We might need to calculate an offset based on the latency/delay 
	
        if (config.scriptstyle):
            logging.printOut("Setting Script Thread for " + node)
            current = NXEThreadScript(nodedescr.contactIP, nodedescr.target, nodedescr.execStepList, pid, nodedescr.connexionssh, timeout, file, sizestring, sleepstring)
        else:
            logging.printOut("Setting Timer Thread for " + node)
        #print sizestring, sleepstring
            current = NXEThreadTimer(nodedescr.contactIP, nodedescr.target, nodedescr.execStepList, pid, nodedescr.connexionssh, timeout, file, sizestring, sleepstring)

        threadList.append(current)

    #Rattrapage for the datelabel stuff
    for i in xrange(0,len(datetab)):
        if not datelabel.has_key(datetab[i]):
            datelabel[datetab[i]] = str(datetab[i])


###
def running(threadList):
    "A function to do stuff as we start the experiment"
    

    #Starting all the threads
    for thread in threadList:
	logging.printOut("Starting Timer Thread for " + thread.location)
        thread.start()

    #Waiting for all the threads to finish
    for thread in threadList:
        logging.printOut("Timer Thread for " + thread.location + " waiting")
        thread.join(thread.timeout)
	logging.printOut("Timer Thread for " + thread.location + " end")
        if not thread.isOK:
            logging.printErr("one or several scripts failed on " + thread.location + ", results may be invalid")

    #Threads all finished -> next step
                              
###
def retrieving_logs(nodeList, archivefolder, timelinelog):
    "A function to locally retrieve all the logs"
    
    import time
    from socket import gethostname;
    hostname = gethostname()
    login = os.getlogin()

    threadList = []
    for node, nodedescr in nodeList.iteritems():

        commandstring = "scp -o CheckHostIP=no -o StrictHostKeyChecking=no /tmp/*_" + pid + "_*.log " + login + '@'  + hostname + ":" + archivefolder + " ; mkdir -p /tmp/old ; mv /tmp/*_" + pid + "_*.log /tmp/old" 
        logging.printOut(commandstring)
	current = NXEThreadCommand(commandstring, node, nodedescr.connexionssh)
        current.start()
	time.sleep(0.05)	
	threadList.append(current)

    for thread in threadList:
        thread.join(200)
	if thread.isOK:
	  pass
	#We don't check the output of this, might be dangerous

    #Copying the timeline to the tempfolder and to the archive path    
    destname = os.path.join(archivefolder, "syslog_runtest.log")
    shutil.copy(timelinelog, destname)
    destname = os.path.join(tmpfolder, "syslog_runtest.log")
    shutil.copy(timelinelog, tmpfolder)

    #Copying everything to the tempfolder for the stats generation stuff
    names = os.listdir(archivefolder)
    for name in names:
        srcname = os.path.join(archivefolder, name)
        shutil.copy(srcname, tmpfolder)

###
def copying_keys(nodeList, user):
    """A Function to copy the ssh keys to the remote nodes

       Here we have to do from the local machine to the remote nodes
       as if the keys are not there, there is no reason why the 
       copy should work without human intervention 

    """
    import time
    from socket import gethostname;
    hostname = gethostname()
    login = os.getlogin()

    
    basecommandstring = "scp -o CheckHostIP=no -o StrictHostKeyChecking=no " + login + '@' + hostname + ":.ssh/* "

    for node, nodedescr in nodeList.iteritems():

        commandstring = basecommandstring + user + '@' + nodedescr.contactIP + ":.ssh/" 
        stdout, stdin, stderr = popen2.popen3(commandstring)
        
        stdout.close()
        stdin.close()
        stderr.close()
        time.sleep(.05)
        

###        
def syncing_scripts(nodeList, scriptpath, user):
    """A function to sync the local scripts to the remote nodes

       Assuming that it is possible to connect from user@node to
       login@hostname (might need to copy the keys first)

       Assuming that the remote bin directory is in the path, etc
       otherwise, it will never work

       
    """
    import time
    from socket import gethostname;
    hostname = gethostname()
    login = os.getlogin()

    threadList = []
    for node, nodedescr in nodeList.iteritems():
        
        commandstring = "mkdir -p bin; scp -o CheckHostIP=no -o StrictHostKeyChecking=no " + login + '@' + hostname + ":" + scriptpath + "/*.sh " + user + "@" + nodedescr.contactIP + ":bin/"   
        #print commandstring
	current = NXEThreadCommand(commandstring, nodedescr.name, nodedescr.connexionssh)
        current.start()
        #Mandatory temporarisation to avoid rejected connections
        time.sleep(.05)
	threadList.append(current)


    for thread in threadList:
        thread.join(200)
        #We don't check the output of this, it might be dangerous



###
def interesting_name(siteList, nodeList):
    """ A function with no name currently that should be used to wrap around the
        parsing of the iperf logs and the generation of the aggregated iperf graphs
    """

    
    for site, sitedescr in siteList.iteritems():

        logging.printOut("Handling logs files for server side " + site) 
        maxtime = -1
        lasttime = 0
        fileparsed = 0
        resulttab = [] 
        completiontab = []
        

        for node, nodedescr in nodeList.iteritems():

            #If we can find the pattern of a sitename in the node target
            #then we are not interested
            #TODO: I don't think it will work with the MPI mode as the target is
            #more like a list
            if nodedescr.target.find(site) == -1:
                continue

            #We need to find the associated filename
            #TODO: check what happens when contactIP != interfaceIP
            #TODO: 
            pattern = tmpfolder + "/*iperf*" + nodedescr.target + "*.log"
            stdout, stdin, stderr = popen2.popen3("ls " + pattern + "| sort -t'_' -k 5 -g")

            filenameList = stdout.readlines()

            stdout.close()
            stdin.close()
            stderr.close()

            for i in xrange(0,len(filenameList)):


                filename = NXEUtils.normalize_whitespace(filenameList[i].replace('\n',''))                
                if (filename == ""):
                    continue

                offsetList = []
                sizeList = []

                if (nodedescr.interdistrib is not None):
                    offsetList = nodedescr.interdistrib.usedtab
                    
                    
                if (nodedescr.sizedistrib is not None):                    
                    sizeList = nodedescr.sizedistrib.usedtab

                #If the offsetList is empty, we need to put the offset of the execution
                #step
                #TODO: Assume there is only one transfer in the file
                #it is very likely to do stupid thing otherwise
                if (len(offsetList) < 1):
                    offsetList.append(float(nodedescr.execStepList[i].date))
                    for i in xrange(1, len(nodedescr.execStepList)):
                        offsetList.append(0.0)
                else:
                    #print repr(offsetList)
                    offsetList[0] = float(offsetList[0]) + float(nodedescr.execStepList[i].date)

                
                if (len(sizeList) < 1):
                    sizeList.append(30000.0) 

                lasttime = NXELogHandling.parse_iperf_result(filename, offsetList, sizeList, resulttab, completiontab, tmpfolder)

                fileparsed += 1

            if (maxtime < lasttime):
                maxtime = lasttime
            
        if (fileparsed > 0):

            filename = str(tmpfolder + "/aggreg_iperf_file_" + site + ".dat")
	    print tmpfolder, filename
            NXEUtils.printResultTabToFile(resulttab, filename, maxtime)
            #TODO: Ooops, what should I put here ?
            NXEChart.drawSumIperfResult( filename, "reno", "Stanford", "")
            
	    pattern = tmpfolder + "/*.eps " + archivefolder
	    stdout, stdin, stderr = popen2.popen3("mv " + pattern)
	    stdout.close()
	    stdin.close()
	    stderr.close()

            #Saving the completion time somewhere
            filename = tmpfolder + "/completiontime_iperf_file_" + site + ".dat"
            NXEUtils.printCompletionTabToFile(completiontab, filename) 

        else:
            logging.printOut("No file parsed for pattern " + site) 



    return

def dump_nodes_law(nodeList):
    """ A function to dump the random values stored in each node
        to keep them in case we need to rerun an experiment 
        with the same values
    """
    
    dumpfile = open( archivefolder + "/nodeslawdump.dat", 'w')

    for node, nodedescr in nodeList.iteritems():
        dumpfile.write(nodedescr.dumpString() + "\n")

    dumpfile.close()



###      
def generating_stats(siteList, datelabel, applicationtype):
    """A function to generate logs and potentially graphs if we know what we are interested in

       We assume here that all logs that are reported at the same date correspond to the same
       experiment and that we are only considering the iperf logs
       TODO rename the method 


    """
    
    #The logs are located in the tmpfolder
    #Basically we need to read the files, take the last line and output the   
    #mean, variance and so on

    sortedfiles = {}
    sortedcompletiontimeresults = {}
    sortedmeangoodputresults = {}
    
    for site, sitedescr in siteList.iteritems():

        #Listing the iperf log files for a given site
        pattern=tmpfolder + "/log_iperf_*" + site + "*.log"
        stdout, stdin, stderr = popen2.popen3("ls " + pattern)

        #if popenfile.stdout == None:
            #there is no output for the command execution 
            #   continue
            
        #sorting the filenames from the list according to the date inside of the scenario
        #For more convient use in the stats the date is substituted by the label used to describe
        #the execution step
        for filename in stdout.readlines():

            stripedfilename=NXEUtils.normalize_whitespace(filename.replace('\n',''))

            #print filename
            splittedfilename = stripedfilename.split('_') 
            date = int(splittedfilename[5])  

            if not sortedfiles.has_key(datelabel[date]):      
                tab = []
                tab.append(stripedfilename)
                sortedfiles[datelabel[date]] = tab
            else:
                sortedfiles[datelabel[date]].append(stripedfilename)


        stdout.close()
        stdin.close()
        stderr.close()

    if len(sortedfiles) == 0:
        #If we had a problem (reservation that finished before retrieval of the files)
        #we won't get any files to read at this stage, so it is pretty useless to continue
        logging.printErr(" no iperf files suitable for analysis, giving up")
        return

    #Okay, here all the files have been sorted
    #We need to do is get the results from the files
    for label, tabfilename in sortedfiles.iteritems():

        #print label, tabfilename
        for i in xrange(0, len(tabfilename)):
            
            resultfile = open(tabfilename[i],'r')  
            linelist = resultfile.readlines()
            resultfile.close()

            if (len(linelist) < 6):
                logging.printOut("Warning, the file " + tabfilename[i] + " is most likely empty (too short)", NXELogging.LOG_WARN) 
                continue
            
            lastline = linelist[len(linelist) - 1] 

            #And now we are ready to parse the output
            #First, we must remove the '-'
            lastline = lastline.replace('-',' ')
            #Then we split the line by removing extra white space
            splittedlastline = NXEUtils.normalize_whitespace(lastline).split(' ')

            #Here we should put some sort of check to prevent cases where the
            #transfert didn't complete well
            #it is the case with iperf if the result we get is 0.0-xx.x 
            beginrange = float(splittedlastline[2])
            completiontime = float(splittedlastline[3])
            meangoodput = float(splittedlastline[7])
            
            #print lastline, splittedlastline, beginrange, completiontime, meangoodput

            if  beginrange == 0.0:
                pass
            else:
                #Warning this transfer didn't complete
                logging.printOut("Warning, the transfer corresponding to file " + tabfilename[i] + " was stopped around " + str(completiontime) +  ": results are very likely to be completely useless, you should really try again",NXELogging.LOG_WARN)
                                
            if label not in sortedmeangoodputresults:
                tab = []
                tab.append(meangoodput)
                sortedmeangoodputresults[label]=tab
            else:
                sortedmeangoodputresults[label].append(meangoodput)

            if label not in sortedcompletiontimeresults:
                tab = []
                tab.append(completiontime)
                sortedcompletiontimeresults[label]=tab
            else:
                sortedcompletiontimeresults[label].append(completiontime)
            

    
    dateorder = []
    sortedstats = {}
    
    #Yeah! the file will contain all the results, the problem is that we have no
    #guarantees with respect with the order in which we get back the stuff
    #so we need to do a 
    for label, tabresult in sortedcompletiontimeresults.iteritems():
        mean = NXEStats.mean(tabresult)
        #variance = NXEStats.variance(tabresult)
        stddev = NXEStats.stdDev(tabresult)
        jainIndex = NXEStats.JainIndex(tabresult)
        #TODO retrieve the bottleneck capacity, one way or the other
        efficiency = NXEStats.efficiency(sortedmeangoodputresults[label], 10000)

        tab = [mean, stddev, jainIndex, efficiency]
        sortedstats[label] = tab
        
        
    for date in datelabel.iterkeys():
        dateorder.append(date)
        
    dateorder.sort()


    resultfile = open(tmpfolder + "/results_iperf.dat",'w')

    resultfile.write("#date\tMean Completion Time\tStdDev of Completion Time\tJainIndex\tEfficiency\n") 

    for i in xrange(0, len(dateorder)):
        tab = sortedstats[datelabel[dateorder[i]]] 
        mean = tab[0]
        #variance = tab[1]
        stddev = tab[1]
        jainIndex = tab[2]
        efficiency = tab[3]
        
        resultfile.write(str(dateorder[i]) + "/" + datelabel[dateorder[i]] + "\t" + str(mean) + "\t" + str(stddev) + "\t" + str(jainIndex) + "\t" + str(efficiency) + "\n") 
    

    resultfile.close()

    #Copying the results in the result in the archive folder as well
    destname = os.path.join(archivefolder, "results_iperf.dat")
    shutil.copy(resultfile.name, destname)

    labeltab = [ "Mean completion time", "Standard Deviation of completion time", "Fairness", "Effiency"]
    NXEChart.drawIperfResult(sortedstats, labeltab, datelabel[0], applicationtype, "results_iperf", tmpfolder) 
    
    destname = os.path.join(archivefolder, "results_iperf.eps")
    srcname = os.path.join(tmpfolder, "results_iperf.eps")
    shutil.copy(srcname, destname)



    

def cleaning(siteList, frontalList, nodeList = {}):
    "A function to clean the mess we have done around here, especially the reservation and local files" 
    
    #Cleaning all reservation
    for site, sitedescr in siteList.iteritems():

        #if site.sshconnexion exists /!\
        if not sitedescr.connexionssh == None:
            command = frontalList[site].resatool + "--delete " + pid 
            sitedescr.connexionssh.exec_command(command)       
            sitedescr.connexionssh.close()

        #Clearing the gnet config if necessary
        if sitedescr.gnet:
            #Call again the same script as before but with the reset switch
            #Anyway script should commit suicide at least when the timer expires
            #We are not keeping the connexion to the gnet so we need to open a new one
            client = SSHClient()
            client.load_system_host_keys()
            #Set the policy to accept automatically new keys
            #Especially useful if we need to connect to a changing pool of nodes like in G5K
            client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            user, host = split_hostname(sitedescr.gnetbox.gnetname)
            client.connect(host, username=user)

            command = sitedescr.gnetbox.gnettool + " " +  sitedescr.gnetbox.gnetparam + " --reset"            
            
            logging.printOut(command)
            stdin, stdout, stderr = client.exec_command(command)


            #TODO: Perhaps we should something with the output 
            #but we don't really need to do anything here



    #Closing all opened connexions on the nodes
    for node, nodedescr in nodeList.iteritems():

        if not nodedescr.connexionssh == None:
            nodedescr.connexionssh.close()
    

    if config.cleanall:
       #Cleaning tmp files
       subprocess.Popen([u"rm",u"-rf", tmpfolder])

           
    logging.printOut("Cleaning ending")    

    exit

def creating_folders(archivepath):
    "A function to create needed additional folder"

    try:

        from datetime import datetime
        now = datetime.today()

        if archivepath == "":
            archivepath = os.getcwd()

        global archivefolder
        archivefolder = archivepath + "/" + now.strftime("%Y/%m/%d/%H-%M-%S")
        archivefolder = os.path.expanduser(archivefolder)
        archivefolder = os.path.normpath(archivefolder)
        #print archivefolder

        os.makedirs(archivefolder)

        #Creating the tmp folder
        global tmpfolder
        tmpfolder = "/tmp/NXE_" + str(pid) 


        os.path.normpath(tmpfolder)

        #print tmpfolder
        #print os.path.exists(tmpfolder)
        
        if not (os.path.exists(tmpfolder)):
            os.mkdir(tmpfolder)
        else:
            logging.printOut("Warning : Tmp folder " + tmpfolder + " already exists -> Removing existing files", NXELogging.LOG_WARN)
            subprocess.call([u"rm",u"-rf", tmpfolder ])
            os.mkdir(tmpfolder)

        
    except EnvironmentError, error: 
        logging.printErr("Unable to create tmp folders : " + repr(error.args))
        #Nothing to clean, yet
        sys.exit()




if __name__ == '__main__':
 
    #currentState = STATE_INIT

    from sys import argv


    if len(argv) <= 1 :
        filename='scenario.xml'
    else:
        filename=argv[1]

    #Here we are supposing that we are somewhere we can reach
    #the rest of the world from.
    #If you are using the GUI, it probably won't be the case and
    #we will need some sort of indirection before
    #as in logging into a G5K access point before doing anything
    #else

    #time = datetime.datetime()

    #Parsing of the input xml file.

    logging.changeState(State.STATE_PARSING)
    logging.printOut("Parsing of file " + filename + " starting")
   

    #outputStream.write(getTime() + ": \n")
    try:
        #print pid
        parsing_config(filename)
        #print pid
    except RuntimeError, error:
        logging.printErr(error.args[0])
        #errorStream.write(getTime() + ": " + error.args[0])
        sys.exit()
        
    logging.printOut("Parsing of file " + filename + " ending")

    #Creating the needed temp folder
    creating_folders(config.archivepath)

    #Saving the config file in the archivefolder, just in case
    destname = os.path.join(archivefolder, "scenario.xml")
    shutil.copy(filename, destname)
    
    nodeList = {}
    #Reservation phase
    logging.changeState(State.STATE_RESERVATION)
    logging.printOut("Reservation of nodes starting")
    handling_reservation(topology.siteList, topology.frontalList, nodeList, config.resadate)
    logging.printOut("Reservation of nodes ending")
    
    #Deployment phase
    logging.changeState(State.STATE_DEPLOY)
    logging.printOut("Deployment of nodes starting")
    handling_deployment(topology.siteList, topology.frontalList, nodeList)
    logging.printOut("Deployment of nodes ending")
    #

    logging.changeState(State.STATE_GNET)
    logging.printOut("Configuration of Gnet starting")
    configure_gnet(topology.siteList, topology.frontalList, config.timeout)
    logging.printOut("Configuration of Gnet ending")

    #IP instanciation of the nodes
    logging.changeState(State.STATE_INSTANCIATING)
    logging.printOut("Instanciation of nodes starting")
    ip_instanciation(scenario.nodeTable, nodeList)
    logging.printOut("Instanciation of nodes ending")


    #########################################################
    #After this point the nodeList is useless
    #Shallow copies will break everything
    nodeList = {}

    if config.key:
        logging.changeState(State.STATE_KEYCOPY)
        logging.printOut("Copying keys starting")
        copying_keys(scenario.nodeTable, config.user)
        logging.printOut("Copying keys ending")

    #Pairing the nodes
    logging.changeState(State.STATE_PAIRING)
    logging.printOut("Pairing of nodes starting")
    try:
        pairing_nodes(scenario.nodeTable, config.user)
    except RuntimeError, error:
        logging.printErr(error.args[0])
        cleaning(topology.siteList, topology.frontalList, scenario.nodeTable)
        sys.exit()
    logging.printOut("Pairing of nodes ending")


    #Syncing the scripts if necessary
    if config.sync:
        logging.changeState(State.STATE_SYNC)
	logging.printOut("Syncing scripts starting")
        syncing_scripts(scenario.nodeTable,config.scriptpath,config.user)
	logging.printOut("Syncing scripts ending")


    #Handling BDTS related stuff if necessary
    if config.BDTS:
        logging.changeState(State.STATE_BDTS)
        logging.printOut("BDTS related stuff starting")
        linking_profile_steps(scenario.nodeTable, profile.nodeTable)
        handling_BDTS(scenario.nodeTable, config.BDTSprogram)
        logging.printOut("BDTS related stuff ending")

    
    threadList = []
    datelabel = {}
    timelinefile = open(config.timelinelog, 'w')
    #Schedule
    logging.changeState(State.STATE_SCHEDULING)
    logging.printOut("Scheduling of nodes starting")
    preparing_schedule(scenario.nodeTable, threadList, config.timeout, timelinefile, datelabel)
    logging.printOut("Scheduling of nodes ending")

    #Running
    logging.changeState(State.STATE_RUNNING)
    logging.printOut("Run of the experiment starting")
    running(threadList)
    logging.printOut("Run of the experiment ending")

    timelinefile.close()

    #Retrieving Logs
    logging.changeState(State.STATE_RETRIEVE)
    logging.printOut("Retrieval of logs starting")
    retrieving_logs(scenario.nodeTable, archivefolder, config.timelinelog)
    dump_nodes_law(scenario.nodeTable)
    logging.printOut("Retrieval of logs ending")
	
    #Generating stuff
    logging.changeState(State.STATE_STATS)
    logging.printOut("Stats and graphs generation starting")
    NXELogHandling.generateBounceDate_graph()
    interesting_name(topology.siteList, scenario.nodeTable)
    generating_stats(topology.siteList, datelabel, config.applicationtype)
    logging.printOut("Stats and graphs generation ending")

    #Cleaning phase
    logging.changeState(State.STATE_FINISHED)
    logging.printOut("Cleaning starting")
    cleaning(topology.siteList, topology.frontalList)
    logging.printOut("Cleaning ending")


    exit
    
    #print filename
    #print config.acces + "/" + config.password + "/" + config.keypath
    #client = paramiko.SSHClient()
    #client.load_system_host_keys()
    #client.connect(config.acces, password = config.password, key_filename = config.keypath )
    #stdin, stdout, stderr = client.exec_command('ls -l')
    #print stdout.read()
    
    #client.close()

    #file.close()
