#!/usr/bin/python
#A module to handle the generation of graphs through the Gnuplot package
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXEChart.py,v 1.6 2008-11-06 11:26:18 rguillie Exp $


#Todo: case when the file used is empty not handled

#import gdchart
import Gnuplot
import os
import string

import NXEUtils
import NXELogging
from NXEUtils import divList, diff, normalize_whitespace, cumulmodification
from NXELogging import logging

#available_color = [ "red", "blue", "green", "yellow", "orange", "black"] 


def drawBounceDateResult(inputfilename, tmpfolder = "/tmp"): 
    """A function to draw a graph from a series of two columns of numbers
       corresponding to the successive date measurements between two hosts
       
    """

    #First, we need to parse the file and transfer it using the 
    head, tail = os.path.split(inputfilename)

    splittedtail = tail.split('_')
    
    source = splittedtail[3]
    dest   = splittedtail[4]

    outputfilename = tmpfolder + "/parsed_" + tail 

    addtab = []
    modtab = []
    #the multab is useless here
    #multab = []
    
    inputfile = open(inputfilename, 'r')
    firstline=inputfile.readline()
    inputfile.close()

    splittedfirstline=normalize_whitespace(firstline).split(" ")

    firstdate=float(splittedfirstline[0])
    colnumber=len(splittedfirstline)

    isWeb100 = (inputfilename.find("web100") != -1)
    
    
    if isWeb100:
        #For the web100, we just need to offset the date/first column

        addtab.append(-1*firstdate)
        modtab.append(0)
        
        for i in xrange(1,colnumber):
            addtab.append(0.0)
            modtab.append(0)
    else:
        #We are likely to have only two columns of floats, but let's stay generical baby!
        #First column
        addtab.append(-1*firstdate)
        modtab.append(0)
        #Second column
        addtab.append(0)
        modtab.append(-1)

        for i in xrange(2,colnumber):
            addtab.append(0.0)
            modtab.append(0)
        
        
    cumulmodification(inputfilename, outputfilename, modtab, addtab)

    gp = Gnuplot.Gnuplot(persist = 1)
    gp('set terminal postscript color enhanced ')
    gp('set xlabel \'Time (s)\'')
    gp('set datafile missing \'-\'')


    if isWeb100:
        gp('set ylabel \'Web100 counter\'')
        gp('set output "' + tmpfolder + "/bounce_date_" + source + "_" + dest + '_web100.eps"')
        usingstring = "1:2 w points t 'CongestionSignals', '' u 1:3 w points t 'PktsRetrans', '' u 1:4 w points t 'Timeouts', '' u 1:5 w points t 'SubsequentTimeouts' " 

    else:
        gp('set ylabel \'Intercommand Time (s)\'')
        gp('set output "' + tmpfolder + "/bounce_date_" + source + "_" + dest + '.eps"') 
        title = "'Bounce date between " + source + " and " + dest + "'"
        usingstring = "1:2 w points t " + title

    
    plot = Gnuplot.PlotItems.File(outputfilename, using=usingstring)
    gp.plot(plot)


def drawSumIperfResult( inputfilename, protocolname, testtype, tmpfolder = "/tmp"):
    """A function to draw graphs using an input file with a series of columns
       Time Size Goodput for each flow

    """

    
    gp = Gnuplot.Gnuplot(persist = 1) 
    #	    gp('set data style lines')
    gp('set terminal postscript landscape color enhanced solid \"Helvetica-Bold\" 16 lw 2.5')
    gp('set key top left')
    gp('set xlabel \'Time (s)\'')
    gp('set ylabel \'Goodput (Mbps)\'')
    gp('set datafile missing \'-\'')
    gp('set yrange [0:1200]')
    gp('set y2range [0:10000]')
    gp('set ytics nomirror')
    gp('set y2tics 1000 ')


    
    inputfile = open(inputfilename, 'r')

    inputfilenamewithoutext, ext = os.path.splitext(inputfilename)
    
    firstline = inputfile.readline()
    #we care only about the first line that is not a comment
    #as it may not contain useful info (like in a comment)
    while ((len(firstline) > 0) and (firstline[0] == '#')):
        firstline = inputfile.readline()
        
    splittedfirstline = firstline.split(' ')
    #print repr(splittedfirstline), len(splittedfirstline) / 3

    inputfile.close()


    usingstring = ""
    sumstring = ""
    for i in xrange(0,len(splittedfirstline) / 3):
        if i == 0:
            usingstring = "1:3 w l notitle"
            sumstring = "1:($3" 
        else:
            usingstring = usingstring + ", '' u " + str(3*i+1) + ":" + str(3*i+3) + "w l notitle"
            sumstring += "+$" + str(3*i+3)

    usingstring = usingstring.replace("notitle", "t \"" + str(len(splittedfirstline)/3) + " Individual " + protocolname + " flows\"", 1)

    sumstring += ") w l t \"" + protocolname + " Aggregate\" axis x1y2"

#    gp('set clip two')
#    gp('set style fill pattern 4 bo')
#    gp('set output "'+ tmpfolder + "/" + filename + '.eps"')
#    gp('set ylabel )
#    gp('set boxwidth 0.9 absolute')
#    gp('set style fill   solid 1.00 border -1')
#    gp('set style histogram clustered gap 1 title  offset character 0, 0, 0')
#    gp('set datafile missing \'-\'')
#    gp('set style data histograms')
#    gp('set xtics border in scale 1,0.5 nomirror rotate by -45  offset character 0, 0, 0')
#    gp('set xtic rotate by -45')


    additionalpath=""
    if ( tmpfolder != ""):
        additionalpath = tmpfolder + "/"

    gp('set output "' + additionalpath + inputfilenamewithoutext + '_indiv.eps"')

    plot = Gnuplot.PlotItems.File(inputfilename, using=usingstring) # with="filledcurves x1")
    gp.plot(plot)

    gp('set output "' + additionalpath + inputfilenamewithoutext + '_sum.eps"')
    plot = Gnuplot.PlotItems.File(inputfilename, using=sumstring) # with="filledcurves x1")
    gp.plot(plot)

    gp('set output "' + additionalpath + inputfilenamewithoutext + '_combi.eps"')
    plot = Gnuplot.PlotItems.File(inputfilename, using=usingstring + ", '' u " + sumstring) # with="filledcurves x1")
    gp.plot(plot)



def drawIperfResult( tabresult , labeltab, reference, testtype, filename, tmpfolder = "/tmp", withReference = True):
    """A function to draw histograms using values coming from a hashtable, using
    a reference to normalize the results
    """
    
    #max = 1.0

    if withReference and not reference in tabresult:
        logging.printErr("reference " + reference + " doesn't exist in the provided result tab " + repr(tabresult))
        return


    gp = Gnuplot.Gnuplot(persist = 1) 
    #	    gp('set data style lines')
    gp('set terminal postscript color enhanced ')
#    gp('set clip two')
#    gp('set style fill pattern 4 bo')
    gp('set output "'+ tmpfolder + "/" + filename + '.eps"')
    gp('set ylabel \'Relative result\'')
    gp('set boxwidth 0.9 absolute')
    gp('set style fill   solid 1.00 border -1')
    gp('set style histogram clustered gap 1 title  offset character 0, 0, 0')
    gp('set datafile missing \'-\'')
    gp('set style data histograms')
    #gp('set xtics border in scale 1,0.5 nomirror rotate by -45  offset character 0, 0, 0')
    #gp('set xtic rotate by -45')


    

    
    xticstring = ""
    
    for i in xrange(0,len(labeltab)):

        if i == 0:
            xticstring = "\"" + labeltab[i] + "\" " + str(i)
        else:
            xticstring = xticstring + ",\"" + labeltab[i] + "\" " + str(i)  

    gp('set xtics   (' + xticstring + ')')

    title = "\"HSTTS Result graph for " + testtype
    if ( withReference ):
        title = title + " with reference " + str(reference)
    title += "\""
    
    gp('set title ' + title)
    #gp('set yrange [ 0.00000 : 300000. ] noreverse nowriteback

    #plot 'immigration.dat' using 6:xtic(1) ti col, '' u 12 ti col, '' u 13 ti col, '' u 14 ti col
    
    #    gp.hardcopy(filename="test.png",terminal="png")    
    #     x = gdchart.Bar()
    #     x.width = 250
    #     x.height = 250
    #     x.xtitle = ""
    #     x.ytitle = "Score"
    #     x.title = "HSTTS Result Graph for " + type 
    #     x.stack_type = "BESIDE"    

    #     x.bg_color = "white"
    
    keytab = []
    datatab = []
    datastring = ""

    

    if (withReference):
        tab=[]
        for i in xrange(0, len(tabresult[reference])):
            tab.append(1.0)            
        datatab.append(tab)
        keytab.append(str(reference))

    for key,tabvalue  in tabresult.iteritems():
        tab = []

        if (withReference):
            if len(tabresult) > 1:
                if key != reference:
                    #print repr(tabvalue), repr(tabresult[reference])
                    tab = divList(tabvalue, tabresult[reference])	   
                else:
                    continue
                #    for i in xrange(0, len(tabvalue)):
                #        tab.append(1.0)

            else:
                tab = tabvalue
        else:
            tab = tabvalue
        
        datatab.append(tab)
        keytab.append(str(key))
        #labeltab.append(str(key))
    	   
    #print datatab[0:len(datatab)]
    #print datastring

    logging.printOut(repr(keytab)+ " " + repr(datatab),NXELogging.LOG_DEBUG)

    datafilename = tmpfolder + "/" + filename + ".dat"
    
    datafile = open(datafilename, 'w')

    title="#Histogram data value comparaison for " + testtype 
    if (withReference):
        title = title + " with reference " + str(reference)
    title+="\n"
    
    #file.write("#Histogram data value comparaison with " +  str(reference) + " \n")

    #First line with the 
    datafile.write("Type\t")
    for i in xrange(0, len(keytab)):
        datafile.write(str(keytab[i]))
        if i < (len(keytab)-1):
            datafile.write("\t")

    datafile.write("\n")

    for i in xrange(0,len(labeltab)):
        datafile.write("\"" + str(labeltab[i]) + "\"" + "\t")
        for j in xrange(0, len(datatab)):      
            datafile.write(str(datatab[j][i]))
            if j < (len(datatab)-1):
              datafile.write("\t")

        datafile.write("\n")

    datafile.close()

    usingstring = ""
    for i in xrange(0,len(datatab)):
        if i == 0:
            usingstring = "2:xtic(1) ti col"
        else:
            usingstring = usingstring + ", '' u " + str(2+i) + " ti col"

#    x.setData(datastring)
    #x.setData(datatab[0], datatab[1], datatab[2])
    #x.setLabels(labeltab)

    #x.setData([20, 100, 80, 30, 50],[30, 110, 90, 40, 60])
    #x.setLabels(["Mon", "Tue", "Wed", "Thu", "Fri"])
    
    #colortab = []
    #for i in xrange(0,len(datatab)):
    #	for j in xrange(0,len(labeltab)):
    #colortab.append(available_color[ i % len(available_color)])
    
    #x.ext_color = colortab

    #Annotation needed for the colors used
    #x.annotate(0, keytab[0], available_color[0])
    #x.annotate(1, keytab[1], available_color[1])
    #x.annotate(2, keytab[2], available_color[2])

    #x.thumbnail = False
    #x.thumblabel = "pouet pouet"
    #x.draw(filename)

    if (withReference):
        gp('set xrange [-1:' + str(len(tabresult[reference]) + 0.5) + ']')
    else:
        gp('set xrange [-1:' + str(len(tabresult[keytab[0]]) + 0.5) + ']')

    #It might be 
    #gp('set yrange [0,' + str(max + ) + ']')

    plot = Gnuplot.PlotItems.File(datafilename, using=usingstring) # with="filledcurves x1")
    gp.plot(plot)


def drawFromFile(filename, outputfile, applicationtype):
    """ A method to generate a chart from the stats generated through the execution of a
        scenario
    
        It assumes that the following file format is used:
        first row contains the description of every column, which will fill the labeltab
        then every row correspond to a result
        the separator used should be the '\t'
    """ 

    file = open(filename, 'r')

    #Parsing first line
    firstline = file.readline()
    firstline = firstline.replace('\n','')
    #Splitting the line
    firstlinesplitted = firstline.split('\t')

    labeltab=[]
    #Filling the labeltab
    #We indeed are starting from 1 as the first column doesn't contain
    #anything interesting
    labeltab= firstlinesplitted[1:]

    #print labeltab


    datatab = {}
    firstlabel = ""
    
    for line in file.readlines():
        tab=[]
        
        linesplitted = line.split('\t')
        #print linesplitted
        #The first item might be something like "date/label"
        #if label contains slashes too, then we will have a good laugh
        datelabel = linesplitted[0].split('/')
        if len(datelabel) >= 2:
            label = datelabel[1]
        else:
            label = datelabel[0]

        if firstlabel == "":
            firstlabel = label

        
        for i in xrange(1, len(linesplitted)):
            tab.append(float(linesplitted[i].replace('\n','')))
            
            
        datatab[label] = tab

    #print datatab
    #print labeltab    
        
    drawIperfResult( datatab, labeltab, firstlabel, applicationtype, outputfile)



#We are also defining a main program to draw the chart 
#from a given file
if __name__ == '__main__':

   #import sys
   #import string

   #print sys.version
   #version = string.split(string.split(sys.version)[0], ".")

   #if version < [ 2, 5 ]:
   #    print pouet

#    tabresult = {}
#    tabresult["Reno"] = [20, 100, 80, 30, 50]
#    tabresult["Bic"] = [30, 110, 90, 40, 60]
#    tabresult["Cubic"] = [40, 150, 40, 40, 70]
#    tabresult["HighSpeed"] = [10, 35, 70, 80, 50]

#    labeltab = [ "Mean", "Variance", "StdDev", "Effiency", "bidule"] 
#    #labeltab = [ "A", "B" ]

#    drawIperfResult( tabresult, labeltab,  "Reno", "BU" , "simple.png")

   #Reading the filename
   from sys import argv

   import os
   
   argc = len(argv)
   #We need something to be able to launch by hand the various functions
   #of this module 

   if argv[1] == "-b":
       drawBounceDateResult(argv[2], os.getcwd())
   #elif argv[1] == "
   else:
    #print argv
    applicationtype = "BU"


    if argc == 2:
        filename = argv[1]
        outputname = filename + ".eps"
    elif argc == 3:
        filename = argv[1]
        outputname = argv[2]
    else:
        filename = argv[1]
        outputname = argv[2]
        applicationtype = argv[3]

    drawFromFile(filename, outputname, applicationtype)

   #Opening the file
   #file = open(filename,'r')

   #for line in file.readlines():

   #    if line[0] = "#":
   #    	  continue



