#! /usr/bin/python
#A module to do all the parsing we need to do on files generated during an experiment
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXELogHandling.py,v 1.5 2008-11-06 11:26:18 rguillie Exp $
#


import string
import os
import popen2

import NXEChart
import NXEUtils
import NXELogging
from NXELogging import logging

###
def parse_iperf_result(inputfilename, offsetList, sizeList, resulttab, completiontab, tmpfolder = "/tmp"):
    """A function to parse the content of a iperf style result file

       offsetList shouldn't be empty

       We are also making the assumption that the flows WILL NEVER be interleaved
       because we won't be able to discriminate from the case with a 

       At the end, we need to even all the transfer length if we want to put them
       together inside a big file 
       Also we need to collect the (size,completion time) here, because we won't be able
       to do it  later
       
       TODO: upgrade iperf to do the following things:
             - uniq keys for each new transfers
             - put a timestamp in front of every new transfer
             - force a way to indicate that a transfer is completed
             - prevent it from changing the unit of the data transfer in the total

    """

    logging.printOut("Handling " + inputfilename)
    
    inputfile = open(inputfilename, 'r')
    linelist = inputfile.readlines()
    inputfile.close()

    if (len(linelist) < 6):
        logging.printOut("Warning, the file " + inputfilename + " is most likely empty (too short)", NXELogging.LOG_WARN)
        #outputStream.write("Warning, the file " + inputfilename + " is most likely empty (too short)\n")
        return
    
    offsetindex = -1
    #lasttime corresponds to the latest date we have encountered in the file
    #eventually offset
    lasttime = 0

    #keyoffset = {}
    #keyresult = {}

    outputstring = ""

    
    ended = True
    currentkey = -1
    nextkey = -1
    currentcompletiontime = 0
    
    for index in xrange(0, len(linelist)):

        currentline=linelist[index]

        #A useful line always contains a '[' at the beginning of the line 
        if ( currentline[0] != "[" ):
            continue

        #Typical UDP line found
        #We should do something more specific for the UDP results..
        if ( currentline.find("out-of-order") != -1 ):
            continue
            
        currentline = currentline.replace('[',' ')
        currentline = currentline.replace(']',' ')
        currentline = currentline.replace('-',' ')
        splittedcurrentline = NXEUtils.normalize_whitespace(currentline).split(' ')        

        if (splittedcurrentline[0] == "SUM"):
            #Shouldn't occur but occurs sometimes when two successive transfers
            #are too close to each other (or a server which don't react too quickly
            #print "Oooops, a SUM, you had interleaved traffic at time " + str(lasttime)
            continue
            
        key = int(splittedcurrentline[0])


        #A line that contains "connected with" means that we have a new transfer
        if ( currentline.find("connected with") != -1 ):

#             #Saving the previous transfer corresponding
#             #THANKS IPERF for your not-uniq keys 
#             if ( key in keyresult ):
#                 resulttab.append(keyresult[key])
               
#             keyresult[key] = ""
             #Getting the corresponding keyoffset
             #print offsetindex, len(offsetList)

             if ( offsetindex < (len(offsetList) - 1)):
                 offsetindex += 1
                 #                keyoffset[key] = offsetindex
             else:
                 #Maybe it is an error, or what ?
                 #outputStream.write("Warning, offsetList too short for " + filename  + " , defaulting to 0\n")
                 offsetindex = 0
                 #keyoffset[key]=0

             if ended:
                 currentkey = key
                 nextkey = -1
                 currentcompletiontime = 0
                 ended = False
                 
                 #print offsetindex
                 initstring = ""
                 #Now the funny part where we generate the empty lines, cause the
                 rangetab = NXEUtils.rangefloat(lasttime, lasttime + NXEUtils.halfround(offsetList[offsetindex]),0.5)

                 for i in xrange(0,len(rangetab)):
                     initstring = initstring + str(rangetab[i]) + " 0 0\n" 

                 lasttime = lasttime + offsetList[offsetindex]
                 #print lasttime, offsetList[offsetindex]

                 outputstring += initstring            
             else:
                 nextkey = key
                 #print "We got a nextkey " + str(nextkey) + " while still working on " + str(currentkey)
             continue

        if (key != currentkey):
            #Hopefully the only case where we have interleaved things
            #is when iperf receives the new transfer a little bit too early
            #and pads to synchronize the two flows (which is annoying)
            #print "Skipping key entry " + str(key) + " at time " + str(lasttime)
            continue

        #Retrieving the measures on a given line
        beginrange = float(splittedcurrentline[1])
        endrange = float(splittedcurrentline[2])
        size    = float(splittedcurrentline[4])
        goodput = float(splittedcurrentline[6])



        #Ok, that won't work for a small range of transfert size, but
        #iperf don't allow us to discriminate between the two events
        if ( (beginrange == 0.0) and (endrange != 0.5)):
            #we need to add the remainder
            #It won't be accurate
            digit = endrange % 1
            if digit < 0.5:
                currentcompletiontime += digit
            else:
                currentcompletiontime += (digit - 0.5)
            
            completiontab.append( ( sizeList[offsetindex % len(sizeList)], currentcompletiontime ) )
            ended = True
            if ( nextkey != -1):
                #print "Switching from " + str(currentkey) + " to " + str(nextkey) + " lasted " + str(currentcompletiontime) 
                currentkey = nextkey
                nextkey = -1
                currentcompletiontime = 0
                ended = False
        else:
            currentcompletiontime += (endrange - beginrange)

        #Some useless conversion here
        string = str(lasttime) + " " + str(size) + " " + str(goodput) + "\n"
        outputstring += string

        lasttime += 0.5

        #keyresult[key] += string
        
    outputstring += str(lasttime) + " 0.0 0.0\n"  
    

    head, tail = os.path.split(inputfilename)
    outputfilename = tmpfolder + "/parsed_" + tail  
    outputfile = open(outputfilename, 'w')

    outputfile.write(outputstring)
    outputfile.flush()
    outputfile.close()

    #resulttab.append(outputfilename)
    resulttab.append(outputstring)


    return lasttime

    #No longer relevant
    #At the end of the loop (ie, the file) we need to put all the result
    #in the resulttab. No duplicates as keyresult[] entries are blanked
    #each time we get the same key again, no guarantees with respect to
    #the order of what we will get

    #for key, keycontent in keyresult:
    #    resulttab.append(keycontent)


def generateBounceDate_graph(tmpfolder = "/tmp"):
    """A function that will be used to generate the Bounce_Date graphs

       The distinction between the Web100 and the actual measurement is done
       at the NXEChart/drawing level.
    """

    pattern = tmpfolder + "/outfile_bounce_date_*.log"
    stdout, stdin, stderr = popen2.popen3("ls " + pattern)

    for filename in stdout.readlines():
        stripedfilename=NXEUtils.normalize_whitespace(filename.replace('\n',''))
        #print stripedfilename
        NXEChart.drawBounceDateResult(stripedfilename, tmpfolder)

    stdout.close()
    stdin.close()
    stderr.close()

    pattern = tmpfolder + "/bounce_date_*.eps"
    stdout, stdin, stderr = popen2.popen3("ls " + pattern)

    for name in stdout.readlines():
        stripedname=NXEUtils.normalize_whitespace(name.replace('\n',''))
        head, tail = os.path.split(stripedname)

        destname = archivefolder + "/" + tail
        shutil.copy(stripedname, destname)

    stdout.close()
    stdin.close()
    stderr.close()

###





if __name__ == '__main__':

    from sys import argv
    import sys
    import re


    if (len(argv) >= 3):
        sitepattern = argv[1]
        startingdate = argv[2]
    elif (len(argv) == 2):
        sitepattern = argv[1]
        startingdate = "0"
    else:
        sitepattern = ""

    resulttab = []
    completiontab = []

    if (os.path.exists('nodeslawdump.dat')):      
        lawfile = open('nodeslawdump.dat','r')
        nodelawlist = lawfile.readlines()
        lawfile.close()
    else:
        logging.printErr("No Law dump file available")
        sys.exit(0)

    maxtime = 0
    fileparsed = 0
 
    for index in xrange(0,len(nodelawlist)):
        
        splittedline = nodelawlist[index].replace('\n','').split('\t')
        #In the old format, we didn't have the corresponding target (ie server, ie log) for a given nodename
        #(ie law distribution)
        if (len(splittedline) < 5):
            #Too short to do anything with it
            #print repr(splittedline), "Too short"
            continue
        
        abstractname = splittedline[0]
        clientname = splittedline[1]
        servername = splittedline[2]
        offsetstring = splittedline[3]
        sizestring = splittedline[4]

        #print abstractname, servername, sitepattern
        

        if (re.compile(sitepattern).search(servername) is None):
            #We don't want to handle files that aren't related to a given site
            #if sitepattern is empty, then all sites are considered
            #print servername + " has no match in " + sitepattern
            continue


        if ((offsetstring.isspace()) and (sizestring.isspace())):
            continue
            #offsetstring = "0"
            #sizestring = "30000"

        pattern = "*iperf*" + servername + "*_" + startingdate + "_*.log"
        stdout, stdin, stderr = popen2.popen3("ls " + pattern)
        filename = stdout.readline()
            
        filename = NXEUtils.normalize_whitespace(filename.replace('\n',''))    
        stdout.close()
        stdin.close()
        stderr.close()

        if (filename == ""):
            continue

        #print "Filling the tabs for " + filename + "[" + offsetstring + "]" + "[" + sizestring + "]" 
        offsettab = NXEUtils.fillTabFromString(offsetstring)
        sizetab = NXEUtils.fillTabFromString(sizestring)

        
        #print "Treating " + repr(offsettab) + repr(sizetab)
        lasttime =  parse_iperf_result(filename, offsettab, sizetab, resulttab, completiontab)
        fileparsed +=1

        if (maxtime < lasttime):
            maxtime=lasttime

        #print maxtime

    #Now we have the max time, we can do the aggregation of all the columns from the tab
    if (fileparsed > 0):

        filename = "aggreg_iperf_file_" + sitepattern + "_" + startingdate + ".dat" 

        NXEUtils.printResultTabToFile(resulttab, filename , maxtime) #, ["A", "B", "C", "D", "E", "F", "G") 

        NXEChart.drawSumIperfResult( filename , "reno", "Stanford")


        #Saving the completion time somewhere
        filename = "completiontime_iperf_file_" + sitepattern + ".dat"

        NXEUtils.printCompletionTabToFile(completiontab, filename) 


    else:
        logging.printOut("No file parsed for pattern " + sitepattern, NXELogging.LOG_WARN)


    #print repr(resulttab)
    #print repr(completiontab)
