#! /usr/bin/python
#A module to redirect the output of everything that is written where we want it 
#to go
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXELogging.py,v 1.2 2008-11-06 11:26:18 rguillie Exp $


import sys
import string
import NXEUtils

from NXEUtils import getTime

LOG_DEBUG = 0
LOG_WARN  = 1
LOG_INFO  = 2

class State:
    """This class describe the possible state in which the NXE process can be

       a.k.a it is an enum

    """ 

    STATE_INIT = 0
    STATE_PARSING = 1
    STATE_RESERVATION = 2
    STATE_DEPLOY = 3
    STATE_SCHED = 4
    STATE_RUNNING = 5
    STATE_RETRIEVE = 6
    STATE_STATS = 7
    STATE_FINISHED = 8
    STATE_PAIRING = 9
    STATE_BDTS = 10
    STATE_SYNC = 11
    STATE_GNET = 12
    STATE_KEYCOPY = 13
    STATE_INSTANCIATING = 14
    STATE_SCHEDULING = 15

class NXELogging:
    """Class to hide what we are actually sending the output, 
       By default sending error to stderr and out to stdout
    """
    
    
    def __init__(self):
        self.currentlevel = LOG_DEBUG
        self.state = State.STATE_INIT
        self.errorStream = sys.stderr
        self.outputStream = sys.stdout

    def __init__(self, level=LOG_DEBUG, out = sys.stdout, error = sys.stderr):
        self.state = State.STATE_INIT
        self.errorStream = error
        self.outputStream = out
        self.currentlevel= level

    def changeState(self, state):
        self.state = state
    
        
    def printState(self):
    
        if self.state == State.STATE_INIT:
            return "Initialisation"
        elif self.state == State.STATE_PARSING:
            return "Parsing files"
        elif self.state == State.STATE_RESERVATION:
            return "Reserving nodes"    
        elif self.state == State.STATE_DEPLOY:
            return "Deploying nodes"
        elif self.state == State.STATE_SCHED:
            return "Generating schedule"
        elif self.state == State.STATE_RUNNING:
            return "Running experiment"
        elif self.state == State.STATE_RETRIEVE:
            return "Retrieving logs"
        elif self.state == State.STATE_STATS:
            return "Generating stats"
        elif self.state == State.STATE_FINISHED:
            return "Finished"
        elif self.state == State.STATE_PAIRING:
            return "Pairing nodes"
        elif self.state == State.STATE_BDTS:
            return "Doing BDTS stuff"
        elif self.state == State.STATE_SYNC:
            return "Syncing script files"
        elif self.state == State.STATE_GNET:
            return "Configuring Gnet"
        elif self.state == State.STATE_KEYCOPY:
            return "Copying SSH keys"
        elif self.state == State.STATE_INSTANCIATING:
            return "Instanciation of nodes"
        elif self.state == State.STATE_SCHEDULING:
            return "Scheduling steps"
        else:
            return "unknown"


    def printOut(self, string, level=LOG_INFO):
        if level >= self.currentlevel:
            #print str(level), str(self.currentlevel)
            self.outputStream.write(getTime() + ": " + self.printState() + ": " + string + "\n")

    def printErr(self, string, level=LOG_INFO):
        if level >= self.currentlevel:
            self.errorStream.write(getTime() + ": " + self.printState() + ": " + string + "\n")
                           

#This is very ugly, but that is the only way to actually shared the 
#logging instance between the several points where it is needed
logging = NXELogging()
