#! /usr/bin/python
#A module containing the XML parsing related stuff
#There should be a file that describe the XML format used in input somewhere

#As we assume that we have a different parser for each kind of function
#and it is very likely that we have collision with the tag names
#do not forget to either split the XML file or put a mechanism to
#only consider relevent tags in the context
#
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXEParser.py,v 1.7 2008-11-06 11:26:18 rguillie Exp $


import string
import NXETypes
import NXEUtils
import NXELogging
from NXELogging import logging

from NXEUtils import * 
from NXETypes import *
from NXERandom import *

from xml.sax import saxutils
from xml.sax import handler
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces

from os.path import expanduser



class TopologyParser(handler.ContentHandler):
    "XML Parsing class to generate the internal topology"

    #This should be enough to represent the topology
    #but it might evolve a little bit before the end
    siteList = {}
    frontalList = {}
    aggregatorList = {}
    linkList = {}
    

    number = 0
    maxmissing = 0
    sitename = ""
    siteabstractname = ""
    gnet = False
    gnetname = ""
    gnettool = ""
    gnetparam = ""
    gnetbox = None
    nodedelay = 0
    nodecapacity = 0
    frontalname = ""
    resatool = ""
    resaparam = ""
    deploytool = ""
    deployparam = ""
    aggregname = ""

    aggregid = ""
    fromname = ""
    fromcapacity = 0
    fromdelay = 0
    
    tagtype = 0
    #Boolean to indicate that we are in the correct context
    inscope = False

    
    def startElement(self, name, attrs):
        if self.inscope:
            if name == 'site':
                self.sitename = ""
                self.siteabstractname = ""
                self.number = 0
                self.maxmissing = 0
                self.gnet = False
                self.gnetname = ""
                self.nodedelay = 0
                self.nodecapacity = 0
                self.aggregname = ""
            elif name == 'gnet':
                self.gnetname = ""
                self.gnettool = ""
                self.gnetparam = ""
                self.gnetbox = None
            elif name == 'frontal':
                self.frontalname = ""
                self.resatool = ""
                self.resaparam = ""
                self.deploytool = ""
                self.deployparam = ""
            elif name == 'aggregator':
                self.aggregid = ""
                #Beware of the shallow copy
                self.linkList = {}
            elif name == 'link':    
                self.fromname = ""
                self.fromcapacity = 0
                self.fromdelay = 0
            elif name == 'sitename':
                self.tagtype = 1
            elif name == 'number':
                self.tagtype = 2
            elif name == 'flag':
                self.tagtype = 3
            elif name == 'nodedelay':
                self.tagtype = 4
            elif name == 'nodecapacity':
                self.tagtype = 5
            elif name == 'aggregname':
                self.tagtype = 6
            elif name == 'frontalname':
                self.tagtype = 7
            elif name == 'resatool':
                self.tagtype = 8
            elif name == 'resaparam':
                self.tagtype = 9
            elif name == 'deploytool':
                self.tagtype = 10
            elif name == 'deployparam':
                self.tagtype = 11
            elif name == 'aggregid':
                self.tagtype = 12
            elif name == 'from':
                self.tagtype = 13
            elif name == 'capacity':
                self.tagtype = 14
            elif name == 'delay':
                self.tagtype = 15
                #Putting this stuff in a separate function won't hurt
            elif name == 'siteabstractname':
                self.tagtype = 16
            elif name == 'maxmissing':
                self.tagtype = 17
            elif name == 'gnetname':
                self.tagtype = 18
            elif name == 'gnettool':
                self.tagtype = 19
            elif name == 'gnetparam':
                self.tagtype = 20
            else:
                self.tagtype = 0
                
        else:
            if name == 'topology':
                self.inscope = True
        
            
    def characters(self, ch):
        if self.inscope:
            if self.tagtype == 1:
                self.sitename = normalize_whitespace(ch)
                
            elif self.tagtype == 2:
                self.number = int(normalize_whitespace(ch),10)
            elif self.tagtype == 3:
                flag = normalize_whitespace(ch)
                if flag == "gnet":
                    self.gnet = True
            elif self.tagtype == 4:
                self.nodedelay = int(normalize_whitespace(ch),10)
            elif self.tagtype == 5:
                self.nodecapacity = int(normalize_whitespace(ch),10)
            elif self.tagtype == 6:
                self.aggregname = normalize_whitespace(ch)
            elif self.tagtype == 7:
                self.frontalname = normalize_whitespace(ch)
            elif self.tagtype == 8:
                self.resatool = normalize_whitespace(ch)
            elif self.tagtype == 9:
                self.resaparam = normalize_whitespace(ch)
            elif self.tagtype == 10:
                self.deploytool = normalize_whitespace(ch)
            elif self.tagtype == 11:
                self.deployparam = normalize_whitespace(ch)
            elif self.tagtype == 12:
                self.aggregid = normalize_whitespace(ch)
            elif self.tagtype == 13:
                self.fromname = normalize_whitespace(ch)
            elif self.tagtype == 14:
                self.fromcapacity = int(normalize_whitespace(ch))
            elif self.tagtype == 15:
                self.fromdelay = int(normalize_whitespace(ch))
            elif self.tagtype == 16:
                self.siteabstractname = normalize_whitespace(ch)
            elif self.tagtype == 17:
                self.maxmissing = int(normalize_whitespace(ch))
            elif self.tagtype == 18:
                self.gnetname = normalize_whitespace(ch)
            elif self.tagtype == 19:
                self.gnettool = normalize_whitespace(ch)
            elif self.tagtype == 20:
                self.gnetparam = normalize_whitespace(ch)

        self.tagtype = 0

    def endElement(self,name):    
        if self.inscope:
            if name == 'site':
                #print self.sitename
                self.siteList[self.sitename] = topologySite(self.sitename, self.number, self.maxmissing, self.gnet, self.gnetbox, self.nodedelay, self.nodecapacity, self.frontalname, self.aggregname, self.siteabstractname)
                self.frontalList[self.sitename] = topologyFrontal(self.sitename, self.frontalname, self.resatool, self.resaparam, self.deploytool, self.deployparam)
            elif name == 'gnet':
                self.gnetbox = topologyGnet(self.gnetname, self.gnettool, self.gnetparam)
            elif name == 'frontal':
                #Nothing to do as it is done when the site tag is closed to avoid problems if the sitename is not given in the right order 
                pass
            elif name == 'aggregator':
                self.aggregatorList[self.aggregid] = topologyAggregator(self.aggregid, self.linkList)
            elif name == 'link':
                self.linkList[self.fromname] = topologyLink(self.fromname, self.fromcapacity, self.fromdelay)
            elif name == 'topology':
                self.inscope = False


class BDTSProfileParser(handler.ContentHandler):
    "XML Parsing class to generate the internal association between a node name and the string corresponding to its bandwith profile, for BDTS usage"

    #A hashtable to maintain the association node <-> profile
    nodeTable = {}

    #The mandatory string to initiate the connection 
    connectstring = "C1 := connect localhost\n"
    #The string used to construct the profile
    outputstring = connectstring
    
    keynb        = 0
    nodename     = ""
    
    #Special treatment for the first date/rate encountered
    firstdate    = True
    firstrate    = True
    
    #Boolean to indicate that we are in the correct context
    inscope = False
    
    def startElement(self, name, attrs):
        
        if self.inscope: 
            if name == 'key':        
                self.tagtype = 1
            elif name == 'date':
                self.tagtype = 2
            elif name == 'rate':
                self.tagtype = 3
            elif name == 'name':
                self.tagtype = 4
            elif name == 'node':
                #New node, reinitialisation of all vars 
                self.tagtype = 0 
                self.outputstring = self.connectstring
                self.firstrate = True
                self.firstdate = True
                self.nodename = ""
            else:
                #We don't care about the other tags
                self.tagtype = 0
        else:              
           if name == 'profile':
                #we are in the scope of a 
                self.inscope = True           
           
           

    def characters(self,ch):
        if self.inscope:
            if (self.tagtype == 1):
                self.keynb = normalize_whitespace(ch)
            elif (self.tagtype == 2):
                self.outputstring = self.outputstring + "[" + normalize_whitespace(ch) + "] "
            elif (self.tagtype == 3):
                self.outputstring += normalize_whitespace(ch) 
            elif (self.tagtype == 4):
                self.nodename = normalize_whitespace(ch)
            #else:
            #    print "Oups, invalid tagtype"
        
        self.tagtype=0

    def endElement(self,name):
        if self.inscope:
            if name == 'date':
                if self.firstdate:
                    self.firstdate = False
                    self.outputstring += "L0 := {C1} limit "
                else:
                    self.outputstring += "modify limit L0 "
            elif name == 'rate':
                if self.firstrate:
                    self.firstrate = False
                    self.outputstring = self.outputstring + " token " + self.keynb
            elif name == 'step':
                self.outputstring += "\n"
            elif name == 'node':
                #Adding the node to the hash-table
                self.nodeTable[self.nodename] = self.outputstring
            elif name == 'profile':
                #End of the profile, getting out of context 
                self.inscope=False


class ConfigParser(handler.ContentHandler):
    """XML parser class to retrieve the general configuration stuff
    Perhaps I will need a Config object to store the info retrieved
    but I am currently not sure that the complete thing outside the
    scope of the wrapper/executor
    A lot of things are still missing here, I think"""
    
    acces = ""
    #flag = ""
    #the max time a client/server should live
    timeout = 0
    keypath = ""
    #Yes, it is ugly to do so, but it isn't used yet 
    password = ""
    user = ""
    #Indicates that we need to sync scripts with every nodes
    sync = False
    #Indicates where the scripts are kept
    scriptpath = "~/NXE/script"
    #Indicates that when we need to care of BDTS related stuff (profile, fc and so on)  
    BDTS = False
    #Indicates the name of the program handling BDTS commands
    BDTSprogram = ""
    #provide a date to start the reservation from
    resadate = ""
    #indicate the kind of application we want to emulate
    applicationtype = "BU"
    repeat = 1
    law = False
    key = False
    scriptstyle = False

    #We can spoof the pid of the NXE instance by using this option,
    #mostly useful for test purposes and reutilisation of a already made reservation
    spoof = -1
    
    cleanall = False
    #Indicates where the logs/archive should be kept
    archivepath = expanduser("~/NXE/archive")
   
    #Indicates where the timelinelog will be written
    timelinelog = expanduser("~/NXE/syslog_runtest.log")

    #Boolean to check that we are in the good scope
    inscope = False
    tagtype = 0

    def startElement(self, name, attrs):

        if self.inscope:
            if name == 'timeout':
                self.tagtype = 1
            elif name == 'flag':
                self.tagtype = 2
            elif name == 'acces': 
                self.tagtype = 3
            elif name == 'keypath':
                self.tagtype = 4
            elif name == 'password':
                self.tagtype = 5
            elif name == 'user':
                self.tagtype = 6
            elif name == 'archivepath':
                self.tagtype = 7
            elif name == 'scriptpath':
                self.tagtype = 8
            elif name == 'BDTSprogram':
                self.tagtype = 9
            elif name == 'spoof':
                self.tagtype = 10
	    elif name == 'timelinelog':
		self.tagtype = 11
	    elif name == 'resadate':
		self.tagtype = 12
	    elif name == 'applicationtype':
		self.tagtype = 13
            elif name == 'repeat':
                self.tagtype = 14
            else:
                self.tagtype = 0
        else:
            if name == 'config':
                self.inscope = True

    def characters(self,ch):    

        if self.inscope:
            if self.tagtype == 1:
                #print normalize_whitespace(ch)
                self.timeout = int(normalize_whitespace(ch),10)
            elif self.tagtype == 2:
                flag = normalize_whitespace(ch).lower()
                if (flag == "bdts"):
                    self.BDTS = True
                elif (flag == "clean"):
                    self.cleanall = True
                elif (flag == "sync"):
                    self.sync = True
                elif (flag == "law"):
                    self.law = True
                elif (flag == "key"):
                    self.key = True
                elif (flag == "scriptstyle"):
                    self.scriptstyle = True
                #self.flag = normalize_whitespace(ch)
            elif self.tagtype == 3:
                self.acces = normalize_whitespace(ch)
            elif self.tagtype == 4:
                self.keypath = expanduser(normalize_whitespace(ch))
            elif self.tagtype == 5:
                self.password = normalize_whitespace(ch)
            elif self.tagtype == 6:
                self.user = normalize_whitespace(ch)
            elif self.tagtype == 7:
                self.archivepath = expanduser(normalize_whitespace(ch)) 
            elif self.tagtype == 8:
                self.scriptpath = expanduser(normalize_whitespace(ch))
            elif self.tagtype == 9:
                self.BDTSprogram = normalize_whitespace(ch)
            elif self.tagtype == 10:
                self.spoof = int(normalize_whitespace(ch),10)
	    elif self.tagtype == 11:
		self.timelinelog = expanduser(normalize_whitespace(ch))
	    elif self.tagtype == 12:
		self.resadate = normalize_whitespace(ch)
	    elif self.tagtype == 13:
		self.applicationtype = normalize_whitespace(ch)
            elif self.tagtype == 14:
                self.repeat = int(normalize_whitespace(ch))
        self.tagtype = 0
        
    def endElement(self, name):

        if self.inscope:
            if name == 'config':
                
             self.inscope = False   


class ScenarioParser(handler.ContentHandler):
    "XML Parser class that construct the execution plan for the scenario"

    number = -1
    nodename = ""
    nodeType = 0
    date = 0
    timeout = 0
    label = ""
    #IP are not instanciated yet, it is not very relevant
    contactIP = ""
    interfaceIP = ""
    scriptname = ""
    scriptparam = ""
    target = ""

    #the thingies needed for having the law distribution retrieved from the config file
    sizedistribname = ""
    sizealpha = 0.0
    sizebeta  = 0.0
    sizetab   = []
    interdistribname = ""
    interalpha = 0.0
    interbeta  = 0.0
    intertab   = []
    
    
    #The list containing the offsets for each execution steps
    #the offsets are added when the nodes are added to the HashTable
    #Yes, it is complicated, but it is better than writing 2000 lines XML files
    offsetList = [] 
    #The list containing the execution steps
    execStepList = []
    #The profile list will be linked later in the process
    profileStepList = []
    
    #A Hashtable containing the information between a node name and a Node Object
    nodeTable = {}

    #Boolean to indicate if we are in the right context
    inscope = False
    tagtype = 0
    

    def startElement(self, name, attrs):

        if self.inscope:
            
            if name == 'node':
                #A new node, cleaning old stuff
                self.number = -1
                self.nodename = ""
                self.execStepList = []
                self.target = ""
                self.nodeType = 0
                self.sizetab = []
                self.sizebeta = 0.0
                self.sizealpha = 0.0
                self.sizedistribname = ""
                self.intertab = []
                self.interbeta = 0.0
                self.interalpha = 0.0
                self.interdistribname = ""
            elif name == 'id':
                self.tagtype = 1
            elif name == 'number':
                self.tagtype = 2
            elif name == 'step':
                #A new step, cleaning old stuff
                self.date = 0
                self.offset = 0
                self.scriptname = ""
                self.scriptparam = ""
                self.label = ""
                self.timeout = 0
            elif name == 'date':
                self.tagtype = 3
            elif name == 'type':
                self.tagtype = 4
            elif name == 'script':
                self.tagtype = 5
            elif name == 'scriptparam':
                self.tagtype = 6
            elif name == 'offset':
                self.tagtype = 7
            elif name == 'target':
                self.tagtype = 8
            elif name == 'label':
                self.tagtype = 9
            elif name == 'sizedistribname':
                self.tagtype = 10
            elif name == 'sizealpha':
                self.tagtype = 11
            elif name == 'sizebeta':
                self.tagtype = 12
            elif name == 'sizetab':
                self.tagtype = 13
            elif name == 'interdistribname':
                self.tagtype = 14
            elif name == 'interalpha':
                self.tagtype = 15
            elif name == 'interbeta':
                self.tagtype = 16
            elif name == 'intertab':
                self.tagtype = 17
            elif name == 'timeout':
                self.tagtype = 18
            else:
                self.tagtype = 0
        else:
            if name == 'scenario':
                self.inscope = True
        
    
    def characters(self,ch):    

        if self.inscope:

            if self.tagtype == 1:
                self.nodename = normalize_whitespace(ch)
            elif self.tagtype == 2:
                self.number = int(normalize_whitespace(ch),10)
            elif self.tagtype == 3:
                self.date = int(normalize_whitespace(ch),10)
            elif self.tagtype == 4:
                self.nodeType = stringToNodeType(normalize_whitespace(ch))
            elif self.tagtype == 5:
                self.scriptname = normalize_whitespace(ch)
            elif self.tagtype == 6:
                self.scriptparam = normalize_whitespace(ch)
            elif self.tagtype == 7:
                self.offsetList.append(int(normalize_whitespace(ch),10))
            elif self.tagtype == 8:
                self.target = normalize_whitespace(ch)
            elif self.tagtype == 9:
                self.label = normalize_whitespace(ch)
            elif self.tagtype == 10:
                self.sizedistribname = normalize_whitespace(ch).lower()
            elif self.tagtype == 11:
                self.sizealpha = float(normalize_whitespace(ch))
            elif self.tagtype == 12:
                self.sizebeta = float(normalize_whitespace(ch))
            elif self.tagtype == 13:
                self.sizetab = normalize_whitespace(ch).split(':')
            elif self.tagtype == 14:
                self.interdistribname = normalize_whitespace(ch).lower()
            elif self.tagtype == 15:
                self.interalpha = float(normalize_whitespace(ch))
            elif self.tagtype == 16:
                self.interbeta = float(normalize_whitespace(ch))
            elif self.tagtype == 17:
                self.intertab = normalize_whitespace(ch).split(':')
            elif self.tagtype == 18:
                self.timeout = int(normalize_whitespace(ch))

        self.tagtype = 0

    def endElement(self, name):

        if self.inscope:
            if name == 'step':
                #Creating a new execstep to be added to the list
                self.execStepList.append(execStep(self.date, self.scriptname, self.scriptparam, self.label, self.timeout))
                #print self.execStepList
            elif name == 'node':    

                if self.nodename == "":
                    raise RuntimeError("A client node has no name, please correct input file\n")

                sizeRandom = None
                interRandom = None
                
                
                splitname = split_basename(self.nodename, self.number)
                splittargetnametab = []
                targetname = ""
                
                #print self.nodename + "/" + str(self.number) + "/" + self.target + "\n"
                if self.nodeType == NXETypes.CLIENT_NODE:
                    if self.target == "":
                        raise RuntimeError("Client node " + self.nodename + " has no target, please correct input file\n")
                    
                    splittargetname = split_basename(self.target, self.number)
                    
                elif self.nodeType == NXETypes.MPI_NODE:
                    if self.target == "":
                        raise RuntimeError("MPI node " + self.nodename + " has no target, please correct input file\n")
                    splitcommatarget = split_comma(self.target)
                    for i in xrange(0, len(splitcommatarget)):
                        #print split_basename(splitcommatarget[i],self.number)
                        splittargetnametab.insert(i,split_basename(splitcommatarget[i],self.number))

                

                for i in xrange(0, len(splitname[1])):                    
                    nodename = splitname[0] + str(splitname[1][i])

                    #It sucks but we need to do so 
                    if self.nodeType == NXETypes.CLIENT_NODE:
                        targetname = splittargetname[0] + str(splittargetname[1][i])
                    elif self.nodeType == NXETypes.MPI_NODE:
                        for i in xrange(0, len(splittargetnametab)):
                            for j in xrange(0,len(splittargetnametab[i][1])):
                                
                                tempname = splittargetnametab[i][0] + str(splittargetnametab[i][1][j])
                                
                                #I don't like this, because we are more likely to create inconsistencies
                                #but that is the only was to force the instanciation of all nodes implied
                                #in the 
                                if tempname not in self.nodeTable:
                                    self.nodeTable[tempname] = Node(tempname, self.contactIP, self.interfaceIP, [], [], "", NXETypes.UNKNOWN)
                                
                                if ((i + j) == 0):
                                    #Initialisation case because of the ','
                                    targetname = tempname
                                else:
                                    targetname = targetname + "," + tempname
                                    
                    else:
                        targetname = ""

                    #/!\ 5 to 1 that there will be some shallow copy in his code that will break everything /!\
                    if ((len(self.offsetList) > 0) and (len(splitname[1]) > 1)):
                        execStepList = offset_execStep(self.execStepList, self.offsetList, i)
                    else:
                        execStepList = self.execStepList

                    sort_execStep(execStepList)    

                    if (self.sizedistribname != "" or len(self.sizetab) > 0 ):
                        #print "Creating sizedistrib for " + nodename
                        sizeRandom = NXErandom(self.sizedistribname, self.sizealpha, self.sizebeta, self.sizetab)

                    if (self.interdistribname != "" or len(self.intertab) > 0):
                        #print "Creating interdistrib for " + nodename
                        interRandom = NXErandom(self.interdistribname, self.interalpha, self.interbeta, self.intertab)
                    
                    
                    if nodename not in self.nodeTable:
                        aNode = Node(nodename, self.contactIP, self.interfaceIP, execStepList, self.profileStepList, targetname, self.nodeType, sizeRandom, interRandom)
		        #print self.execStepList
                        self.nodeTable[nodename] = aNode
                    else:
                        #if (self.nodeTable[nodename].nodeType != NXETypes.UNKNOWN ):
                        logging.printOut("Warning: node " + nodename + " was defined previously.", NXELogging.LOG_WARN)
                        #    self.nodeTable[nodename].execStepList.append(self.execStepList)
                        #    print repr(self.execStepList)
                        #else:
                        #    print "Warning: node " + nodename + " was defined previously with type " + str(self.nodeType) + "/" + str(self.nodeTable[nodename].nodeType) + "\n"
                            


                    #Uncomment if necessary but I don't think it is a good idea
                    #If we do that, scenarios where a number of client/server are missing
                    #might go as far as the execution even though they are not consistant
                    #if self.nodeType != NXETypes.MPI_NODE:
                    #   if targetname not in self.nodeTable:
                    #       self.nodeTable[targetname] = Node(targetname, self.contactIP, self.interfaceIP, None, None, "", NXETypes.UNKNOWN)  

            elif name == 'scenario':
                self.inscope = False

