#! /usr/bin/python
#A module to generate the random numbers we may need during an experiment
#or alternatively faking it using a set of numbers 
#
#TODO: the way we retrieve the tabs from the config 
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXERandom.py,v 1.3 2008-11-06 11:26:18 rguillie Exp $



import string
import random

import NXELogging
from NXELogging import logging

def getRandom(method, alpha, beta):
    """A function to call the corresponding method in the Python random library
       currently it isn't checking anything (consistence of alpha and beta
       with respect to law distribution chosen)
    """

    if ( method == "uniform" ):
        return random.uniform(alpha,beta)
    elif ( method == "gauss" or method == "normal" ):
        return random.gauss(alpha, beta)
    elif ( method == "exp" ):
        return random.expovariate(1/alpha)
    elif ( method == "lognorm" ):
        return random.lognormvariate(alpha,beta)
    elif ( method == "pareto" ):
        #Not using the pareto function defined by the python library as it doesn't take the mean into account
        xm = (alpha - 1)/(alpha) * beta
        #print xm
        return int(round(xm/pow(random.uniform(0.0,1.0),(1/alpha))))

        #return random.paretovariate(alpha)
    elif ( method == "weibull" ):
        return random.weibullvariate(alpha,beta)
    else:
        return random.random()


class NXErandom:
    "A class to generate the required random numbers for an experiment"

###
    def __init__(self):
        "Empty initialisation"
        self.method = ""
        self.alpha = 0
        self.beta = 0
        self.tab = []
        self.index = -1
        self.usedtab = []
        
    def __init__(self, method, alpha, beta = 0, tab = []):
        "Normal Initialiation"
        self.method  = method
        self.alpha   = alpha
        self.beta    = beta
        self.tab     = tab
        self.index   = -1
        self.usedtab = []
            
###
    def toString(self):
        "Method to print nicely the object"

        outputstring="using random "

        if ( self.method != "" ):
            outputstring = outputstring + self.method + " (" + str(self.alpha) + "," + str(self.beta) + ") used " + repr(self.usedtab)  
        else:
            outputstring = outputstring + repr(self.tab) + " used " + repr(self.usedtab)
        
        return outputstring

###

    def dumpString(self):
        "Method to put the random values used separated by ':'"

        outputstring=""
        for i in xrange(0,len(self.usedtab)):
            if i == 0:
                outputstring = str(self.usedtab[i])
            else:
                outputstring = outputstring + ":" + str(self.usedtab[i])
                
        return outputstring
    
    
###
    def getRandom(self):
        "A method to retrieve a random number corresponding to a size"

        if (self.method == ""):
            #We need to use the tab instead
            if ( len(self.tab) > 0):
                #Due to the initial value, we need to pre-incr the index
                #We are supposing it loops when we reach the end
                self.index = ((self.index + 1) % len(self.tab))

                self.usedtab.append(float(self.tab[self.index]))
                return float(self.tab[self.index])
                
            else:
                #Should we raise an error or a warning ?
                return 0

        else:
            #We call some generic function to do the random
            value = getRandom(self.method, self.alpha, self.beta)
            self.usedtab.append(value)
            return value
        



# if __name__ == '__main__':

    

#      aleat = NXErandom("pareto",15.0, 5000.0,  tab = [1,2,3,4])

#      print aleat.toString()

#      index = 0
#      while (index < 15):
#         print "Test NXErandom " + str(aleat.getRandom())
#         index = index +1
#      exit

#      print aleat.toString()
