#!/usr/bin/python
# A module that contains all the useful functions to calculate
# stats relevant for our purpose
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>

# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXEStats.py,v 1.2 2008-11-06 11:26:18 rguillie Exp $


from math import sqrt

def mean(valuetab):
    "A function to calculate the mean of values provided in a table"

    mean = 0

    for i in xrange(0,len(valuetab)):
        mean += valuetab[i]

    mean = mean / len(valuetab)

    return mean

def variance(valuetab):
    "A function to calculate the variance of values provided in a table"

    meanvalue = mean(valuetab)
    vartab =[]
    for i in xrange(0,len(valuetab)):
        vartab.append((valuetab[i] - meanvalue) * (valuetab[i] - meanvalue))
        
    return mean(vartab)
    

def stdDev(valuetab):
    "A function to calculate the standard deviation of values provided in a table"
    return sqrt(variance(valuetab))

def JainIndex(valuetab):
    "A function to calculate the Jain index on values provided in a table"

    meanvalue = mean(valuetab)
    
    squaresum = 0
    for i in xrange(0,len(valuetab)):
        squaresum += (valuetab[i] * valuetab[i]) 

    return ((meanvalue * meanvalue) * len(valuetab)) / squaresum
    
def efficiency(valuetab, bottleneckcapacity):
    """A function to calculate the efficiency of a transfer

       We should divide everything by the corresponding bottleneck capacity 

    """

    sum = 0
    
    for i in xrange(0, len(valuetab)):
        sum += valuetab[i] 
        
    return sum/bottleneckcapacity
    
