#! /usr/bin/python
#A module containing the thread related stuff needed
#by the NXE programm
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXEThread.py,v 1.7 2008-11-06 11:26:18 rguillie Exp $

import paramiko
from threading import Thread
from NXEUtils import getTime, script_execStep

import NXELogging
from NXELogging import logging

class NXEThreadConnecting(Thread):
    """A class to optimize the opening of the remote ssh connexion on the
       nodes by using threads
    """
    
    #The symbolic name of the node so that we can find it back in the list
    name = ""
    #The destination of the connexion
    contactIP = "localhost"
    #The paramiko sshclient connexion to be opened
    connexion = None
    user = ""

    def __init__(self, contactIP, name, user):
        Thread.__init__(self)
        self.name = name 
        self.contactIP = contactIP
        self.connexion = None
        self.user = user

    def run(self):
        self.connexion = paramiko.SSHClient()
        self.connexion.load_system_host_keys()
        self.connexion.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        if self.user != "":
            self.connexion.connect(self.contactIP, username=self.user)
        else:
            self.connexion.connect(self.contactIP)        
        #It would be great if we had some return code


class NXEThreadCommand(Thread):
    """A class to optimize command execution by using threads
       It is assumed that the output of the command will be
       OK! or KO! (yes, it is a convention) 
    """

   

    #A string containing the command to execute with its parameters
    commandstring = ""
    #A string containing the name of the site/node on which the command is executed
    #Mainly used for debugging issues
    location = ""
    #The paramiko sshclient connexion to execute the command on the remote site
    connexion = None
    output = ""
    

    def __init__(self, commandstring, location, connexion ):
        Thread.__init__(self)
        self.commandstring = commandstring
        self.location = location
        self.connexion = connexion
        self.isOK = False 
        self.output = ""

    def run(self):

        if (self.connexion is not None):
            stdin, stdout, stderr = self.connexion.exec_command(self.commandstring)
        else:
            self.isOK = False
            logging.printErr("Unable to connect to " + self.location)
            return
        
        string = stdout.read()
        error =  stderr.read()
        #print self.location, string
	#print self.location, error
	

        if string == "OK!\n":
            self.isOK = True
        else:
            #The rest of the stdout can be interesting stuff that can be forwarded to the upper level 
            self.isOK = False
            splittedstring=string.split('\n')
            self.output=splittedstring[1:len(splittedstring)]
            
        stdin.close()
        stdout.close()
        stderr.close()
	#That could have been a good idea if kadeploy wasn't sending junk 
	#in stderror as well
        #if len(error) > 0:
        #    self.isOK = False

class NXEThreadScript(Thread):
    """A thread class that generates a shell script according to a execStepList
       and that is copied at the beginning of a run on a given node to be executed
       
    """
    
    #A list of execution steps to perform
    execStepList = []

    #Where the thread is corresponding with (debug stuff)
    location = ""
    target = "localhost"
    #The paramiko sshclient connexion to execute the command on the remote site
    connexionssh = None


    #The size string, it is supposed to be a string of volumes to transfer with
    #their unit separated by hyphens XXXU:YYYU: 
    sizestring = ""
    #The intersleep string, it is supposed to be a string of sleep durations
    #separated by hyphens
    sleepstring = ""


    #The pid of the calling NXE process
    pid = ""
    #A offset in case we see that we need to start earlier i.e we have
    #long latency
    offset = 0
    timeout = 700
	
    scriptfile=""
    scriptpath=""

    isOK = True
    #stdoutList = []
    #stderrList = []
   

    def __init__(self, location, target, execStepList, pid, connexion, timeout, file, sizestring = "", sleepstring = "", offset = 0):

        Thread.__init__(self)
        self.location = location
	if target != "":
	   self.target = target
        self.connexionssh = connexion
	self.execStepList = execStepList
        self.pid = pid
        self.timeout = timeout
        self.offset = offset
        self.sizestring = sizestring
        self.sleepstring = sleepstring

	#A shared file by all the threads to write the timeline (hopefully) of the experiment
	self.logfile = file

	#Forget to do that the python interpreter will make everything go wrong
	self.stdoutList = []
	self.stderrList = []

	#print str(len(execStepList))+ "/" + str(len(self.execStepList))

        self.scriptpath="/tmp/NXE_" + self.pid + "/"
        self.scriptfile=self.location + "_" + self.target + ".sh"

        extraoptions=""
        #TODO: Temporay hack, the interval and size should be integrated
        #even with the "classical" iperf client
        if (self.sleepstring != "" ):
            extraoptions=" --interval " + self.sleepstring 

        if (self.sizestring != "" ):
            extraoptions=extraoptions + " --size " + self.sizestring  
        

        script_execStep(self.execStepList, self.scriptpath + self.scriptfile, self.pid, self.location, self.timeout, self.target, extraoptions, self.offset)


    def run(self):
        
        from os import getlogin
        from socket import gethostname
        hostname = gethostname()
        login = getlogin()

        if (self.connexionssh is None):
            logging.printErr("Unable to connect to " + self.location)
            self.isOK = False
            return

        #Copying and then executing the script on the remote location
        commandstring = "scp -o CheckHostIP=no -o StrictHostKeyChecking=no " + login + '@' + hostname + ":" + self.scriptpath + self.scriptfile + " ." 

        commandstring = commandstring + "; chmod +x " + self.scriptfile + "; ./" + self.scriptfile
         

        stdin, stdout, stderr = self.connexionssh.exec_command(commandstring)
        stdin.close()
         
        self.logfile.write(getTime() + " : " + self.location + " connects to " + self.target + " with script " + self.scriptpath + self.scriptfile)
        logging.printOut(self.location + " starts using script " + self.scriptpath + self.scriptfile, NXELogging.LOG_DEBUG) 
         
        countwait = len(self.execStepList)
         
        string =""
        for string in stdout.readlines():
            if string == "OK!\n": 
                countwait -= 1
            else:
                self.isOK = False
            if countwait <= 0:
                #All steps accounted for
                self.logfile.write(getTime() + " : all steps completed for node" + self.location) 
                break
            logging.printOut(self.location + " out [" + string + "]", NXELogging.LOG_DEBUG)


        for string in stderr.readlines():
            logging.printOut(self.location + " err [" + string + "]", NXELogging.LOG_DEBUG)

        stdout.close()
        stderr.close()


class NXEThreadTimer(Thread):
    """A thread class that recreates a Timer class but that can be used to
       execute a series of events on a given node
    """
    
    
    #A list of execution steps to perform
    execStepList = []

    #Where the thread is corresponding with (debug stuff)
    location = ""
    target = "localhost"
    #The paramiko sshclient connexion to execute the command on the remote site
    connexionssh = None


    #The size string, it is supposed to be a string of volumes to transfer with
    #their unit separated by hyphens XXXU:YYYU: 
    sizestring = ""
    #The intersleep string, it is supposed to be a string of sleep durations
    #separated by hyphens
    sleepstring = ""


    #The pid of the calling NXE process
    pid = ""
    #A offset in case we see that we need to start earlier i.e we have
    #long latency
    offset = 0
    timeout = 700
	


    isOK = True
    stdoutList = []
    stderrList = []
   

    def __init__(self, location, target, execStepList, pid, connexion, timeout, file, sizestring = "", sleepstring = "", offset = 0):

        Thread.__init__(self)
        self.location = location
	if target != "":
	   self.target = target
        self.connexionssh = connexion
	self.execStepList = execStepList
        self.pid = pid
        self.timeout = timeout
        self.offset = offset
        self.sizestring = sizestring
        self.sleepstring = sleepstring

	#A shared file by all the threads to write the timeline (hopefully) of the experiment
	self.logfile = file

	#Forget to do that the python interpreter will make everything go wrong
	self.stdoutList = []
	self.stderrList = []

	#print str(len(execStepList))+ "/" + str(len(self.execStepList))

        

    
    def run(self):

        #The executionStep are sorted, so we just need to iterate on the list
	import time
        curTimer = 0

        if (self.connexionssh is None):
            logging.printErr("Unable to connect to " + self.location)
            self.isOK = False
            return

        
	#print self.location + " " + str(curTimer) + " " + str(len(self.execStepList))

        extraoptions=""
        #TODO: Temporay hack, the interval and size should be integrated
        #even with the "classical" iperf client
        if (self.sleepstring != "" ):
            extraoptions=" --interval " + self.sleepstring 

        if (self.sizestring != "" ):
            extraoptions=extraoptions + " --size " + self.sizestring  


        for i in xrange(0, len(self.execStepList)):

            
            timeToSleep = self.execStepList[i].date - curTimer - self.offset
            timeBeforeSleep = time.time()
	    timeAfterSleep = timeBeforeSleep

            #A loop to ensure that we sleep a least the required time
            #Disable if too expensive
            while (timeToSleep > (timeAfterSleep - timeBeforeSleep) and timeToSleep > 0.1 ) :
                timeToSleep -= (timeAfterSleep - timeBeforeSleep)  
                timeBeforeSleep = time.time() 
                time.sleep(timeToSleep)
                timeAfterSleep = time.time()
		#print timeToSleep

            curTimer = self.execStepList[i].date
            
            #The "local" timeout (in a given step) is applied if it is defined
            #otherwise, we use the "
            timeoutoption = ""
            if self.execStepList[i].timeout > 0:
                timeoutoption = " --timeout " + str(self.execStepList[i].timeout)
            else:
                timeoutoption = " --timeout " + str(self.timeout) 

            commandstring = self.execStepList[i].scriptname + " --pid " + str(self.pid) + " --date " + str(self.execStepList[i].date) + " --target " + self.target + extraoptions + timeoutoption + " " + self.execStepList[i].scriptparams 
            logging.printOut(self.location + " " + str(curTimer) + " " + commandstring,NXELogging.LOG_DEBUG)
            #getTime() + " " + self.location + " " + str(curTimer) + " " + commandstring
	    self.logfile.write(getTime() + " : " + self.location + " connects to " + self.target + " with " + self.execStepList[i].scriptname + " : " + str(curTimer) + "\n")  
	    stdin, stdout, stderr = self.connexionssh.exec_command(commandstring)
            #it is not blocking as long as we don't try to read the stdout
            stdin.close()
            self.stdoutList.append(stdout)
	    self.stderrList.append(stderr)

	#print self.location + " " + str(curTimer) + " " + str(len(self.execStepList))

	#This bit doesn't work because the scripts don't seem to send back the right trace
        for i in xrange(0, len(self.execStepList)):
	    #print i,self.location
	
	    string=""
	    for string in self.stdoutList[i].readlines():
	    	logging.printOut(self.location + " out [" + string + "]", NXELogging.LOG_DEBUG)
                self.isOK = self.isOK and (string == "OK!\n")

	    #print string
	    for string in self.stderrList[i].readlines():
                logging.printOut(self.location + " err [" + string + "]", NXELogging.LOG_DEBUG)

	    
	    #self.isOK = True
	    self.stdoutList[i].close()
	    self.stderrList[i].close()

        
        
               
