#! /usr/bin/python
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: NXETypes.py,v 1.5 2008-11-06 11:26:18 rguillie Exp $

import string
import paramiko


UNKNOWN=0
SERVER_NODE=1
CLIENT_NODE=2
BCK_NODE=3
MPI_NODE=4

def stringToNodeType(text):
    """Converting a string to its corresponding nodetype,
    assuming that the extra whitespaces have already been removed
    """
    lowertext=text.lower()

    
    if lowertext == "server":       return SERVER_NODE
    elif lowertext == "client":     return CLIENT_NODE
    elif lowertext == "background": return BCK_NODE
    elif lowertext == "mpi":        return MPI_NODE
    else: return UNKNOWN
    

def nodeTypeToString(nodetype):
    "Printing the Node type"
    if nodetype == SERVER_NODE:   return "server"
    elif nodetype == CLIENT_NODE: return "client"
    elif nodetype == BCK_NODE:    return "background"
    elif nodetype == MPI_NODE:    return "mpi"
    else: return ""




#
class Node:
    "A class to define the Object type Node, basically containing its name, how to contact it and what should be done with it"
    
####
    def __init__(self):
        "Empty initialisation"
        self.name = ""
        self.contactIP = ""
        self.interfaceIP = ""
        self.nodeType = UNKNOWN
        self.target = ""
        self.connexionssh = None
        self.execStepList = []
        self.profileStepList = ""
        self.sizedistrib = None
        self.interdistrib = None

####        
    def __init__(self, name, contactIP, interfaceIP, execStepList , profileStepList, target, nodeType, sizedistrib = None, interdistrib = None):
        self.name = name         
        self.contactIP = contactIP
        self.nodeType = nodeType
        self.interfaceIP = interfaceIP
        self.execStepList = execStepList
        self.profileStepList = profileStepList
        self.target = target
        self.connexionssh = None
        self.sizedistrib = sizedistrib
        self.interdistrib = interdistrib

####        
    def toString(self):
        "Method to print nicely the object"

        outputstring = self.name + " (" + self.contactIP + "/" + nodeTypeToString(self.nodeType) + " ): " + self.interfaceIP + " targetting " + self.target + " \n"
        
        for execStep in self.execStepList:
            outputstring = outputstring + "\t" + execStep.toString()

        outputstring += "\n"
        
        outputstring += self.profileStepList    
        #for profileStep in self.profileStepList:    
        #    outputstring = outputstring + "\t" + profileStep.toString()

        outputstring = outputstring + "Size distribution " + self.sizedistrib.toString()
        outputstring = outputstring + " Interstarttime distribution " + self.interdistrib.toString()
        
        return outputstring     

####
    def dumpString(self):
        "Method to dump the random value used"

        outputstring = self.name + "\t" + self.contactIP + "\t" + self.target + "\t"

        if self.interdistrib is not None:
            outputstring += self.interdistrib.dumpString()
        else:
            outputstring += " "
        outputstring += "\t"

        if self.sizedistrib is not None:
            outputstring += self.sizedistrib.dumpString()
        else:
            outputstring += " "
         
        return outputstring

#    
class execStep:
    "Class representing a step of a scenario to execute"
    
    def __init__(self):
        self.date = 0
        self.scriptname = ""
        self.scriptparams = ""
        self.label = ""
        self.timeout = 0
        
    def __init__(self, date, scriptname, scriptparams, label, timeout=0):
        self.date = date
        self.scriptname = scriptname
        self.scriptparams = scriptparams
        self.label = label
        self.timeout = timeout

    def toString(self):
        outputstring = "t=" + str(self.date) + "/timeout" + self.timeout + " : " + self.scriptname + " " + self.scriptparams + " (" + label + ")\n" 
        return outputstring

    
    
#
class profileStep:

    def __init__(self):
        self.date = 0
        self.stepValue = 0

    def __init__(self, date, stepvalue):
        self.date = date
        self.stepValue = stepvalue

    def toString(self):
        outputstring = "t=" + self.date + " : " + self.stepValue + " Mbps"
        return outputstring

#
class topologySite:

    def __init__(self):
        self.sitename = ""
        self.siteabstractname = ""
        self.number = 0
        self.maxmissing = 0
        self.gnet = False
        self.gnetbox = None
        self.nodedelay = 0
        self.nodecapacity = 0
        self.frontalname = ""
        self.aggregname = ""
        self.connexionssh = None

    def __init__(self, sitename, number, maxmissing, gnet, gnetbox, nodedelay, nodecapacity, frontalname, aggregname, abstractname):
        self.sitename = sitename        
        self.number = number
        self.maxmissing = maxmissing
        self.gnet = gnet
        self.gnetbox = gnetbox
        self.nodedelay = nodedelay
        self.nodecapacity = nodecapacity
        self.frontalname = frontalname
        self.aggregname = aggregname
        self.connexionssh = None
        
        if abstractname == "":
            self.siteabstractname = sitename
        else:
            self.siteabstractname = abstractname

    def toString(self):
        outputstring = self.sitename + ", a.k.a. " + self.abstractname + " ( gnet=" + self.flag + " ): " + self.number + " nodes, " + self.nodedelay + "ms delay, " + self.nodecapacity + "Mbps. Administrated by " + self.frontalname + ", linked to " + self.aggregname + "\n"  
        return outputstring

#
class topologyFrontal:

    def __init__(self):
        sitename = ""
        frontalname = ""
        resatool = ""
        resaparam = ""
        deploytool = ""
        deployparam = ""

    def __init__(self, sitename, frontalname, resatool, resaparam, deploytool, deployparam):
        self.sitename = sitename
        self.frontalname = frontalname
        self.resatool = resatool
        self.resaparam = resaparam
        self.deploytool = deploytool
        self.deployparam = deployparam

    def toString(self):
        outputstring = self.frontalname + " ( " + self.sitename + " ): " + self.resatool + " " + self.resaparam + " / " + self.deploytool + " " + self.deployparam + "\n"   
        return outputstring

#
class topologyAggregator:

    def __init__(self):       
        aggregname = ""
        linkList = {}

    def __init__(self, aggregname, linkList):
        self.aggregname = aggregname
        self.linkList = linkList

    def toString(self):
        outputstring = self.aggregname + " : " 

        for site, link in self.linkList.iteritems():
            outputstring = outputstring + site + " : " + link.linkcapacity + "Mbps, " + link.linkdelay + "ms\n"
        
        return outputstring
        
class topologyLink:

    def __init__(self):
        linkname = ""
        linkcapacity = 0
        linkdelay = 0

    def __init__(self, linkname, linkcapacity, linkdelay):
        self.linkname = linkname
        self.linkcapacity = linkcapacity
        self.linkdelay = linkdelay

    def toString(self):
        outputstring = self.linkname + " : " + self.linkcapacity + "Mbps, " + link.linkdelay + "ms\n"

        return outputstring

class topologyGnet:
    
    #Probably not a good idea because we need maybe also the latency that is define on the link  
    def __init__(self):
        gnetname = ""
        gnettool = ""
        gnetparam = ""

    def __init__(self, gnetname, gnettool, gnetparam):
        self.gnetname = gnetname
        self.gnettool = gnettool
        self.gnetparam = gnetparam

    def toString(self):
        outputstring = self.gnetname + " : " + self.gnettool + " " + self.gnetparam + "\n"
    
