#!/bin/bash
#A simple script that execute a remote 'date' command through ssh repetitevely 
#to measure the impact of congestion on pseudo-interactive traffic 
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: bounce_date.sh,v 1.2 2008-11-06 11:26:39 rguillie Exp $



help()
{
cat <<EOF

A script that execute a remote 'date' command through ssh repetitively
Usage: $0 [option value]

   -h|--help: display this and exit
   --pid: the pid of the calling NXE process
   -tg|--target: the target node to use
           (default: $_target)
   -o|--output: name of the output file to be used 
   -t|--timeout: the max time before the script commits suicide
           (default: $_timeout)
EOF

echo "KO!"

exit 0
}

#Setting default options

_target="localhost"
_pid=0
_date=0
_timeout=700


elapsed=0
lasttime=0
outputfile=""


#Handling the options

while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	--pid)
	    shift
	    _pid=$1
	    shift
	;;
	-d|--date)
	    shift
	    _date=$1
	    shift
        ;;
        -t|--timeout)
	    shift
	    _timeout=$1
	    shift
	;;
	-tg|--target)
	    shift
	    _target=$1
	    shift
        ;;
	-o|--output)
	    shift
	    outputfile=$1
	    shift
	;;
	*)
	    echo "$1: Unknown option"
	    help
	;;
    esac
done

if [ -z "$outputfile" ]; then

    outputfile="/tmp/outfile_bounce_date_"`hostname`"_"$_target"_"$_pid"_"$_date
    
    outputfileweb100=$outputfile"_web100.log"
    outputfile=$outputfile".log"

    rm $outputfile
    rm $outputfileweb100
fi 

#Main loop

next_web100=`get_next_connexion_id.sh`


while [ $elapsed -lt $_timeout ]; do

        currentdate=`date +%s.%N`
        measureddate=`ssh $_target date +%s.%N`

	web100string=`readvar $next_web100 CongestionSignals PktsRetrans Timeouts SubsequentTimeouts | xargs | awk '{printf "%u %u %u %u", $3, $6, $9, $12}'`


        echo "$currentdate $measureddate" >> $outputfile
	echo "$currentdate $web100string" >> $outputfileweb100

        if [ $elapsed -eq 0 ]; then 
	    elapsed=1
	    lasttime=`echo $currentdate | cut -d'.' -f 1`
	else
	    currenttime=`echo $currentdate | cut -d'.' -f 1`
	    elapsed=`expr $elapsed + $currenttime - $lasttime`
	    lasttime=$currenttime
	fi

	next_web100=`expr $next_web100 + 1`
	
done

echo "OK!"
exit 1