#!/bin/bash
#
#
#A script to perform the change of congestion control method used during the 
#forthcoming test
#
#Assumes that we provide the name of the module (tcp_westwood) or the
#corresponding congestion control method name (westwood)
#
#Normally when we unload a module, the congestion control fallbacks to 
#the most recent previously method used (module or not) but it would
#be risky to use this property 

#This should launched by root or through sudo
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: change_congestion_control.sh,v 1.2 2008-11-06 11:26:39 rguillie Exp $





#First clean up a little by removing the already loaded tcp_modules
modules_list=`lsmod | grep "tcp_" | cut -f 1 -d' ' `

for module in $modules_list; do
#    echo $module
    rmmod $module
done



#Switching with the protocol 
case $1 in
    reno|tcp_reno)
    #we don't need to load a module in this case, it's built-in 
	echo reno > /proc/sys/net/ipv4/tcp_congestion_control
    ;;
    bic|tcp_bic)
    #we don't need to load a module in this case, it's built-in
	echo bic > /proc/sys/net/ipv4/tcp_congestion_control
    ;;
    #Remember to add specific cases if 
    #a given method needs extra-variables modification
    #and don't forget to clean up
    *)	
	modprobe tcp_$1 || modprobe $1
	echo $1 | cut -d'_' -f2 > /proc/sys/net/ipv4/tcp_congestion_control
    ;;
esac

#End
