#!/bin/bash
#A script to configure the 1G bottleneck in the Lyon site (event though there is some sort of genericity because it might be useful one day)
#
#Should be run as root, as we need to modify the networking interface (obviously)
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: config_1g_bottleneck.sh,v 1.2 2008-11-06 11:26:39 rguillie Exp $




help()
{
cat <<EOF

A script to configure the 1G bottleneck in the Lyon site, hopefully without having to remove completely the main interface
Usage: $0 [option value]


  -h|--help: display this and exit
  -i|--interface: name of the interface used for the bottleneck
       (default: $_interface )
  -m|--main-interface: name of the main interface
       (default: $_maininterface )
  -d|--dns: name of the DNS server
       (default: $_dns )
  -o|--origin: name of the origin of the connexion to avoid losing the SSH connexion while reconfiguring
       (default: $_origin )
  -ip|--ip-offset: number by which we should offset the IP for the new interface
       (default: $_ip_offset )
  -gw|--gateway: name of the gateway
       (default: $_gateway )
  -n|--netmask: netmask to be used to configure the bottleneck
       (default: $_netmask )
  -s|--site: name of the target site from which we can deduce the dns and gw
       (default: $_site )
  -t|--timeout: automatic timeout to reset the interface after a given time
  -r|--reset: resetting the interface to its previous state


EOF
exit 0

}

#Default options
_interface="eth0"
_maininterface="eth1"
_dns="ldap.lyon.grid5000.fr"
_origin="rork.lyon.grid5000.fr"
_gateway="gw.lyon.grid5000.fr"
_site="lyon"
_ip_offset=100 
_reset=0
_netmask="255.255.248.0"
_timeout=0

#Handling the options
while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	-i|--interface)
	    shift
	    _interface=$1
	    shift
	;;
	-m|--main-interface)
	    shift
	    _maininterface=$1
	    shift
	;;
	-d|--dns)
	    shift
	    _dns=$1
	    shift
	;;
	-o|--origin)
	    shift
	    _origin=$1
	    shift
	;;
	-ip|--ip-offset)
	    shift
	    _ip_offset=$1
	    shift
	;;
	-gw|--gateway)
	    shift
	    _gateway=$1
	    shift
	;;
	-n|--netmask)
	    shift
	    _netmask=$1
	    shift
	;;
	-s|--site)
	    shift
	    _site=$1
	    _dns=$_site".grid5000.fr"
	    _gateway="gw."$_site".grid5000.fr"
	    shift
        ;;
	-r|--reset)
	    shift
	    _reset=1
	;;
	-t|--timeout)
	    shift
	    _timeout=$1
	    shift
	;;
	*)
	     echo "$1: Unknown option"
	    help
	;;
     esac
done

#getting the main interface IP adress
ipadress=`ifconfig $_maininterface | head -n 2 | tail -n 1 | cut -f12 -d' ' | cut -f2 -d':'`
lastbit=`echo $ipadress | cut -d'.' -f4`
newipadress=`echo $ipadress | cut -d'.' -f-3`"."`expr $lastbit + $_ip_offset`

network=`echo $ipadress | cut -d'.' -f1-2`".0.0"


#If 
if [ "$_reset" -eq "1" ]; then

#Resetting the routes
route add -net $network netmask $_netmask dev $_maininterface
route add default gw $_gateway dev $_maininterface

#Removing the interface 
ifconfig $_interface down 


else

#Starting the new interface and trying to get a IP adress on it

if [ `ifconfig $_interface up > /dev/null 2> /dev/null; echo $?` -gt 0 ]; then
    echo "$_interface doesn't exist"
    exit -1
fi

if [ `dhclient $_interface > /dev/null 2> /dev/null; echo $?` -gt 0 ]; then
    echo "Unable to get an IP automatically for $_interface" 
fi


if [ `ifconfig $_interface | grep -q "inet"; echo $?` -gt 0 ]; then    
    #Second try, this time we set the IP ourselves 
    ifconfig $_interface down
    if [ `ifconfig $_interface $newipadress $_netmask > /dev/null 2> /dev/null; echo $?` -gt 0 ]; then
	echo "Failing to set the IP $newipadress on $_interface" 
    fi
fi 

newipadress=`ifconfig $_interface | head -n 2 | tail -n 1 | cut -f12 -d' ' | cut -f2 -d':'`
newnetwork=`echo $newipadress | cut -d'.' -f1-2`".0.0"

#Setting the necessary route to avoid to lose the connexion
route add -host $_origin dev $_interface
route add -host $_dns dev $_interface

route del -net $network  netmask $_netmask dev $_maininterface
route del default gw $_gateway dev $_maininterface

route add default gw $_gateway dev $_interface

if [ $_timeout -gt 0 ]; then
#Timeout is set, we need to reconfigure the interface after having slept for a period of time
    sleep $_timeout
   
#Resetting the routes
    route add -net $network netmask $_netmask dev $_maininterface
    route add default gw $_gateway dev $_maininterface

#Removing the interface 
    ifconfig $_interface down 
    

fi


fi