#!/bin/bash
#A script to configure the vlans for the Gnet-10 on the Lyon site
#
#Should be run as root, as we need to modify the networking interface (obviously)
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: config_vlan.sh,v 1.2 2008-11-06 11:26:40 rguillie Exp $


help()
{
cat <<EOF

A script to configure a vlan
Usage: $0 [option value] 


  -h|--help: display this and exit
  -i|--interface: name of the interface we want to change
       (default: $_interface )
  -v|--vlan: number of the vlan we want to use
       (default: $_vlan )
  -o|--ip_offset: number by which we should offset the IP for the vlan
       (default: $_ip_offset )
  -m|--mac_prefix: prefix we need to put for the MAC adress in the vlan
       (default: $_mac_prefix )
  -n|--netmask: netmask
       (default: $_netmask )
  -gw|--gateway: name of the gateway
       (default: $_gateway ) 
  -r|--reset: Resetting the vlan



EOF
exit 0

}

#Default options
_vlan=102
_interface="eth1"
_mac_prefix="08:00:48"
_ip_offset=100
_netmask="255.255.248.0"
_gateway="gw.lyon.grid5000.fr"
_reset=0

#Handling the options

while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	-i|--interface)
	    shift
	    _interface=$1
	    shift
	;;
	-v|--vlan)
	    shift
	    _vlan=$1
	    shift
	;;
	-m|--mac_prefix)
	    shift
	    _mac_prefix=$1
	    shift
	;;
	-o|--ip_offset)
	    shift
	    _ip_offset=$1
	    shift
	;;
	-n|--netmask)
	    shift
	    _netmask=$1
	    shift
	;;
	-gw|--gateway)
	    shift
	    _gateway=$1
	    shift
	;;
	-r|--reset)
	    shift
	    _reset=1
	;;
	*)
	    echo "$1: Unknown option"
	    help
	;;
     esac
done

#Loading the 802.1q module
modprobe 8021q


#getting the interface IP adress
ipadress=`ifconfig eth1 | head -n 2 | tail -n 1 | cut -f12 -d' ' | cut -f2 -d':'`
lastbit=`echo $ipadress | cut -d'.' -f4`
newipadress=`echo $ipadress | cut -d'.' -f-3`"."`expr $lastbit + $_ip_offset`

#Getting the interface MAC adress 
macadress=`ifconfig eth1 | head -n 1 | cut -f11 -d' '`
newmacadress=$_mac_prefix":"`echo $macadress | cut -d':' -f4-` 

interfacevlan=$_interface"."$_vlan

#TODO: should calculate the network properly using the netmask
#Awfully annoying to do that in shell script
network=`echo $newipadress | cut -d'.' -f1-2`".0.0"


if [ "$_reset" -eq "1" ]; then

#Resetting the routes
route add -net $network netmask $_netmask dev $_interface
route add default gw $_gateway dev $_interface
route del default gw $_gateway dev $interfacevlan

#Deleting the vlan
vconfig rem $interfacevlan

else

#Creating the interface with the vlan
vconfig add $_interface $_vlan


#Changing the IP adress
ifconfig $interfacevlan $newipadress netmask $_netmask
 
#Changing the MAC adress
ifconfig $interfacevlan down
ifconfig $interfacevlan hw ether $newmacadress
ifconfig $interfacevlan up


#Modification of the default route 
route del -net $network netmask $_netmask dev $_interface
route add default gw $_gateway dev $interfacevlan
route del default gw $_gateway dev $_interface

fi