#!/bin/bash
#A simple script to handle a deployment on G5K using kadeploy
#
#
#Should be compliant with OAR 2.0
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: deploy_g5k.sh,v 1.2 2008-11-06 11:26:40 rguillie Exp $


help()
{
cat <<EOF

Usage $0 [option value]

   -h|help: display this and exit
   -pi|--pid: the pid of the programm calling this script to allow multiple simultaneous usage
   -e|--environment: the name of the kernel image to deploy
   
EOF

echo "KO!"
exit 0
}

#Default parameter value
_pid=""
_environment="debian4all-web100-2.6.22"
_cluster=""
_login=`whoami`
#_partition="sda3"

#Handling the options
while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
#	-p|--partition)
#	    shift
#	    _partition=$1
#	    shift
#	;;
	-pi|--pid)
	    shift
	    _pid=$1
	    shift
	;;
	-e|--environment)
	    shift
	    _environment=$1
	    shift
	;;
	-c|--cluster)
	    shift
	    _cluster=$1
	    shift
	;;
	-l|--login)
	    shift
	    _login=$1
	    shift
	;;    
	*)
	    echo "$1: Unknown option"
	    help
	;;
    esac

done


#
file_node="/tmp/"`ls /tmp | grep "uniq_"$_pid"_"$_cluster`

#If there is more than one file, it won't work..
node_number=`cat $file_node | wc -l`




node_number_margin=`expr $node_number + 10`

 
output_kadeploy="/tmp/output_kadeploy_"$_pid"_"$_cluster

#Starting the kadeploy process
ssh -tt `hostname` kadeploy -e $_environment -l $_login -f $file_node > $output_kadeploy


#Ugly check, there is no other way, as the output of kadeploy is very nice

error_number=`cat $output_kadeploy | tail -n $node_number_margin | grep "error" | wc -l `


if [ $error_number -gt 0 ]; then

    #Oups, it seems that there was some problems during the deploy phase 
    
    #Try again once for nodes in error
    nodes_in_error="/tmp/nodes_in_error_"$_pid"_"$_cluster
    cat $output_kadeploy | tail -n $node_number_margin | grep "error" | awk '{print $1}' > $nodes_in_error

    ssh -tt `hostname` kadeploy -e $_environment -f $nodes_in_error > $output_kadeploy
    
    error_number=`cat $output_kadeploy | tail -n $node_number_margin | grep "error" | wc -l `
    

    if [ $error_number -gt 0 ]; then
	#removing the invalid deployed nodes
	cat $output_kadeploy | tail -n $node_number_margin | grep "error" | awk '{print $1}' > $nodes_in_error
	cat $file_node | grep -v -F "`cat $nodes_in_error`" > $file_nodes 
	

	echo "KO!"
	cat $nodes_in_error
	exit -1
    fi

fi

echo "OK!"
exit 0