#!/bin/bash
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET, Ludovic HABLOT 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# Ludovic HABLOT             - <ludovic.hablot@ens-lyon.fr> 
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: exec_pingpong.sh,v 1.2 2008-11-06 11:26:40 rguillie Exp $


help()
{
cat <<EOF
Usage : $0 [mpichmad|gridmpi|openmpi|mpich|tcp|mpich-instr] [distant|local] {power}"

EOF

echo "KO!"
exit 0
}

DEBUG=0
TAU=0

if test $# -lt 2 -o $# -gt 3
then
	help
fi

if test $DEBUG -eq 1
then
	FILE=/tmp/mpi.data
else
	FILE="`date +%F_%T`-$MPI_IMPL-$LOC.data"
fi

echo "[Exec_pingpong.sh: Begin]" > $FILE

MPI_IMPL=$1
LOC=$2
puissance=$3

OMPI_LOC="/home/lyon/jcmignot/"
site_distant="rennes.grid5000.fr"

if test "$MPI_IMPL" = "mpich-instr"
then
	MPICH2="mpich2-install-instr"
	MPI_IMPL="mpich"
else
	MPICH2="mpich2-install"
fi

if test $DEBUG -eq 1
then
	NB_EXCH=200
else
	NB_EXCH=200
fi


if test $LOC = "distant" -a $MPI_IMPL = "mpichmad"
then
	ARGS="-l $NB_EXCH -e"
else
	ARGS="-l $NB_EXCH -i"
fi	
FIRST_NODE=$(head -n 1 node_file)
LAST_NODE=$(tail -n 1 node_file)

if test $TAU -eq 1
then
	ssh $FIRST_NODE "rm tautrace.*.trc events.*.edf profile.*"
	ssh $LAST_NODE "rm tautrace.*.trc events.*.edf profile.*"	
fi

case $MPI_IMPL in
	mpichmad)
    	MPIRUN="$HOME/soft/mpich-mad-install/bin/mpirun"
    	if test $LOC = "distant"
    	then
    		MPI_ARGS="-config ~/config_files/channels_double_4.cfg"
	        export MPI_NET_FILE="config_files/networks_double_4.cfg"
	    else
	        MPI_ARGS="-config ~/config_files/channels_simple_2.cfg"
    	    export MPI_NET_FILE="config_files/networks_simple_2.cfg"
	    fi;;
	openmpi)
    	MPIRUN="$OMPI_LOC/OMPI-build/bin/mpirun"
    	if test $LOC = "distant"
    	then
			MPI_ARGS="-np 4 -machinefile node_file"
    	else
        	MPI_ARGS="-np 2 -machinefile node_file -mca btl_tcp_sndbuf 4194304 -mca btl_tcp_rcvbuf 4194304"
        	#MPI_ARGS="-np 2 -machinefile node_file -mca btl_tcp_sndbuf 4194304 -mca btl_tcp_rcvbuf 4194304 -mca btl_tcp_eager_limit 67108864"
        	#MPI_ARGS="-mca btl_tcp_frag_size 32768 -np 2 -machinefile node_file -mca btl tcp"
    	fi;;
	gridmpi)
    	#MPIRUN="$HOME/soft/gridmpi/bin/gridmpirun"
    	MPIRUN="$HOME/soft/gridmpi/bin/mpirun"
   		#if test $LOC = "local"
    	#then
        	MPI_ARGS="-np 2 -c node_file"
    	#fi
		;;
	mpich)
		MPIRUN="$HOME/soft/$MPICH2/bin/mpirun"
		if test $LOC = "distant"
		then
			MPI_ARGS="-n 4"
		else
			MPI_ARGS="-n 2"
		fi
		if test $LOC = "distant"
		then
			scp node_file $site_distant:
		fi
		ssh $FIRST_NODE "~/soft/$MPICH2/bin/mpdboot $MPI_ARGS -f node_file"
		if test $DEBUG -eq 1
		then
			ssh $FIRST_NODE  "~/soft/$MPICH2/bin/mpdtrace"
		fi
		;;
	tcp)
		echo 0 > /dev/null
		;;
	*)
		echo $1: Unknown option
    	help
esac

#version ludo
REP_PROG=~/soft/pingpong/
PROGNAME=testmpibench

#version mpich-madeleine
#REP_PROG=~/soft/mpich-mad/examples/benchmarks/
#PROGNAME=mad_para_ping


echo "" > $FILE
if test $DEBUG -eq 1
then
	if test "$puissance" = ""
	then
		#tailleMsg=16777216
		#puissance=0
		#tailleMaxMsg=17000000
		#tailleMsg=1
		#puissance=0
		puissance=20
		#tailleMaxMsg=2
		#tailleMsg=1048576
		#puissance=0
		#tailleMaxMsg=1500000
	fi
   	tailleMsg=$(echo "2^$puissance" | bc )
	tailleMaxMsg=$(expr $tailleMsg + 1)
else
	tailleMsg=1
	puissance=0
	tailleMaxMsg=70000000
	#tailleMsg=4194304
	#puissance=22
	#tailleMaxMsg=5000000
	#tailleMsg=16777216
	#puissance=24
	#tailleMaxMsg=17000000
fi


while test $tailleMsg -lt $tailleMaxMsg
do
    echo "Envoi de $ARGS mesg de $tailleMsg"
    
    if test "$LOC" = "distant" -a "$MPI_IMPL" = "gridmpi"
    then
    
    	ADDR_PORT_FILE="/tmp/addr_port.data"
    	
		echo "./soft/gridmpi/bin/impi-server -server 2 > $ADDR_PORT_FILE &"
    	./soft/gridmpi/bin/impi-server -server 2 > $ADDR_PORT_FILE &
		sleep 1
		ADDR=$(cat $ADDR_PORT_FILE | cut -d':' -f1)
		PORT=$(cat $ADDR_PORT_FILE | cut -d':' -f2)
		echo "$ADDR:$PORT"
    	MPI_ARGS_1="-client 0 $ADDR:$PORT -np 1 -c node_file"
		MPI_ARGS_2="-client 1 $ADDR:$PORT -np 1 -c node_file"
   		echo "ssh oar.$site_distant $MPIRUN $MPI_ARGS_2 $REP_PROG$PROGNAME -s $tailleMsg $ARGS &"
        echo "eval  $MPIRUN $MPI_ARGS_1 $REP_PROG$PROGNAME -s $tailleMsg $ARGS"
	
   		ssh $site_distant "$MPIRUN $MPI_ARGS_2 $REP_PROG$PROGNAME -s $tailleMsg $ARGS" 1>/dev/null &
        valeur=$(eval  "$MPIRUN $MPI_ARGS_1 $REP_PROG$PROGNAME -s $tailleMsg $ARGS")	
		if test $DEBUG -eq 1
		then
			echo "$valeur"
		fi
		valeur=$(echo "$valeur" | grep MESURE | sed "s/MESURE : //")
    else
        if test $MPI_IMPL = "tcp"
        then
            portno=$(expr 9000 + $puissance)
            cmd="~/soft/latenceTcp/client $LAST_NODE $portno $NB_EXCH $tailleMsg"
            #lauching server
            if test $DEBUG -eq 1
            then
                echo ssh $LAST_NODE "~/soft/latenceTcp/server $portno $tailleMsg &"
            fi
            ssh $LAST_NODE "~/soft/latenceTcp/server $portno $tailleMsg" &
            ssh $LAST_NODE "ps ax"
        else
	        cmd="$MPIRUN $MPI_ARGS $REP_PROG$PROGNAME -s $tailleMsg $ARGS"
		fi
        if test $DEBUG -eq 1
		then
			echo $cmd
		fi
		#particular cases
		if test $MPI_IMPL = "mpich" -o $MPI_IMPL = "tcp"
		then
			valeur=$(ssh $FIRST_NODE $cmd)
		else
			valeur=$(eval $cmd)
		fi
		if test $DEBUG -eq 1
		then
			echo "$valeur"
		fi
		valeur=$(echo "$valeur" | grep MESURE | sed "s/MESURE : //")
    fi
    echo $puissance $valeur >> $FILE
    tailleMsg=$(expr $tailleMsg \* 2);
    puissance=$(expr $puissance + 1);

done

if test $DEBUG -eq 1
then
	cat $FILE
fi

if test $MPI_IMPL = "mpich"
then
	ssh $FIRST_NODE "~/soft/$MPICH2/bin/mpdallexit"
fi

if test $TAU -eq 1
then
	date=$(date +%F_%T)
    ssh lyon.grid5000.fr "mkdir tau_results/$MPI_IMPL/$date/"
	for node in $FIRST_NODE $LAST_NODE
	do
		ssh $node "scp tautrace.* oar.lyon.grid5000.fr:tau_results/$MPI_IMPL/$date/"
		ssh $node "scp events.* oar.lyon.grid5000.fr:tau_results/$MPI_IMPL/$date/"
		ssh $node "scp profile.* oar.lyon.grid5000.fr:tau_results/$MPI_IMPL/$date/"
	done
	scp $FILE lyon.grid5000.fr:tau_results/$MPI_IMPL/$date/$MPI_IMPL.data
fi

echo "[Exec_pingpong.sh: End]" > $FILE

echo OK!

exit 0
