#!/bin/bash
#A script to generate the logs and graphs from the sar output using gnuplot
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: gnuplot_cpu_monitor.sh,v 1.2 2008-11-06 11:26:40 rguillie Exp $


help()
{
cat <<EOF

A script to generate the logs and graphs from the sar output using gnuplot
Usage: $0 [option value]


   -h|--help: display this and exit
   -i|--input: the name of the file containing the sar output
               (default: $_input_filename ) 
   -o|--output: the name of the output file to which write the resulting graph
               (default: $_output_filename)
   -ie|--image_extension: indicates the image extension to be used by gnuplot for output
                     (default: $_image_extension)
   -if|--image_format: indicates the 'set terminal' option to be used by gnuplot for output
                     (default: $_image_format)

EOF

exit 0
}

#Setting default options

_input_filename="cpu_monitor"
_output_filename="gnuplot_cpu_monitor.eps"
_image_extension="eps"
_image_format="postscript landscape color solid \"Helvetica-Bold\" 16 lw 2.5"
#_image_format="postscript landscape"


while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	-i|--input)
	    shift
	    _input_filename=$1
	    shift
        ;;
	-o|--output)
	    shift
	    _output_filename=$1
	    shift
	;;
	-ie|--image_extension)
	    shift
	    _image_extension=$1
	    shift
	;;
	-if|--image_format)
	    shift
	    _image_format=$1
	    shift
	;;
	*)
	    echo "$1: Unknown opion"
	    help
	;;
    esac

done


date=`cat $_input_filename | head -n 1 | cut -f2`
time=`cat $_input_filename | head -n 3 | tail -n 1 | cut -f1 -d' '`

epoch=`date -d "$date $time" +%s` 

length=`cat $_input_filename | wc -l` 
length=`expr $length - 3`

cat $_input_filename | tail -n $length > /tmp/cpu_monitor.$$

awk -v epoch=$epoch 'BEGIN { i = epoch; } { print i, $3, $5, $8 ; i += 1; }' /tmp/cpu_monitor.$$ > /tmp/gnuplot_cpu_monitor.$$


gnuplot_command="set key top left; set yrange [0:100]; plot '/tmp/gnuplot_cpu_monitor.$$' u 1:2 w l t 'User', '/tmp/gnuplot_cpu_monitor.$$' u 1:3 w l t 'System', '/tmp/gnuplot_cpu_monitor.$$' using 1:(\$2+\$3) w l t 'Sum'" 

echo "set xlabel \"T (s)\" ; set ylabel \"CPU (%)\" ; set terminal $_image_format ; $gnuplot_command" | gnuplot > $_output_filename


rm /tmp/*.$$