#!/bin/bash
#A simple script to launch a client (basically a iperf client) 
#
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: launch_client.sh,v 1.3 2008-11-06 11:26:40 rguillie Exp $


help()
{
cat <<EOF

A script to launch a client for the NXE
Usage: $0 [option value]

 
   -h|--help: display this and exit
   --pid:  the pid of the calling NXE process
   -d|--date: the date at which the process is supposed to start
                 (default: $_date ) 
   -t|--timeout: the max time before the script commits suicide
                 (default: $_timeout )
   -b|--baseport: the base port to use for the server
                 (default: $_baseport)
   -v|--volume : indicates the volume of data to send
                 (default: $_volume )
   -c|--continuous : indicates that are sending continously till the timeout occurs
                 (default: $_continuous) 
   -p|--parallel : indicates the number of parallel streams to send the data
                 (default: $_parallel)
   -w|--window : the size of the TCP buffer to use
                 (default: $_buffer_size)
   -u|--udp :    using UDP
                 (default: $_udp)  
   -bd|--bandwidth: (UDP only)  
                 (default: $_bandwidth )
   -tg|--target: the target server to use
                 (default: $_target )
   -pr|--protocol: the name of the TCP protocol to use
                 (default: $_protocol )     
   -g|--gnet: indicates that the vlan for the gnet should be configured
                 (default: $_gnet)
   -l|--limit: indicates a strict bandwith limit
                 (default: $_limit )
   -m|--mode: type of bandwidth limitation performed
                 (default: $_mode )
EOF

echo "KO!"

exit 0
}

#Setting default options

_timeout=700
_volumeunit="30000M"
_baseport=5001
_parallel=1
_continuous=0
_buffer_size="4M"
_udp=0
_pid=0
_date=0
_target="localhost"
_protocol="reno"
_gnet=0
_mode=""
_limit=""

_path=`pwd`"/bin"

#Handling the options

while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	--pid)
	    shift
	    _pid=$1
	    shift
	;;
	-d|--date)
	    shift
	    _date=$1
	    shift
	;;
	-t|--timeout)
	    shift
	    _timeout=$1
	    shift
	;;
	-b|--baseport)
	    shift
	    _baseport=$1
	    shift
	;;
	-v|--volume)
	    shift
	    _volume=$1
	    shift
	;;
	-c|--continuous)
	    shift
	    _continuous=1
	;;
	-p|--parallel)
	    shift
	    _parallel=$1
	    shift
        ;;
	-w|--window)
	    shift
	    _buffer_size=$1
	    shift
        ;;
	-u|--udp)
	    shift
	    _udp=1
	    #shift
	;;
	-bd|--bandwidth)
	    shift
	    _bandwidth=$1
	    shift
	;;
	-tg|--target)
	    shift
	    _target=$1
	    shift
	;;
	-pr|--protocol)
	    shift
	    _protocol=$1
	    shift
        ;;
	-g|--gnet)
	    shift
	    _gnet=1
	;;
	-l|--limit)
	    shift
	    _limit=$1
	    shift
	;;
	-m|--mode)
	    shift
	    _mode=`echo $1|tr '[:lower:]' '[:upper:]'`
	    shift
	;;
	--path)
	    shift
	    _path=$1
	    shift
	;;
	*)
	    #echo "$1: Unknown option"
	    #help
	    shift
	;;
    esac
done

#Changing the protocol

sudo $_path/change_congestion_control.sh $_protocol > /dev/null


#First the switches
if [ "$_gnet" -eq "1" ]; then
	    sudo $_path/config_vlan.sh > /dev/null
fi


if [ -n "$_mode" ]; then
    sudo $_path/limit_bandwidth.sh --target $_target --limit $_limit --mode $_mode
fi


options="-c $_target" 


if [ "$_udp" -eq "1" ]; then
    options=$options" -u -b "$_bandwidth
fi


if [ "$_continuous" -eq "1" ]; then

    options=$options" -t $_timeout"

else

    #Separing unit and volume 
    unit=`echo $_volumeunit | sed 's/[0-9]*//' `
    volume=`echo $_volumeunit | sed 's/[M,K]$//'`

    #Splitting the volume between the parallel streams and adding back the unit
    volume=`expr $volume / $_parallel`
    _volumeunit=$volume$unit

    options=$options" -n "$_volumeunit

fi


#Remains of the global options
options=$options" -w "$_buffer_size

#echo $options

#base_port=`expr $_baseport + $_date`
currentport=$_baseport

index=1

while [ $index -le $_parallel ]; do


    iperf $options -p $currentport > "/tmp/plouf"$index & 
    currentport=`expr $currentport + 1`
    index=`expr $index + 1`
    #keeping a string with all the pids forked
    lpid=$lpid" "$!

done

#sleeping for the timeout
sleep $_timeout
#killing all the pids forked
kill -9 $lpid > /dev/null
kill -9 $lpid > /dev/null

if [ -n "$_mode" ]; then
    sudo $_path/limit_bandwidth.sh --reset --target $_target --mode $_mode
fi

#First the switches
if [ "$_gnet" -eq "1" ]; then
	    sudo $_path/config_vlan.sh --reset > /dev/null
fi


echo "OK!"
exit 1 
