#!/bin/bash
#A script to launch a background client
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: launch_client_background.sh,v 1.3 2008-11-06 11:26:40 rguillie Exp $



help()
{
cat <<EOF

A script to launch a background client using D-ITG
Usage: $0 [option value]

     -h|--help: display this and exit
     -t|--timeout: the max time before the script commits suicide
                (default: $_timeout )  
     -d|--date: the date at which the process is supposed to start
                 (default: $_date )
     --pid: the pid of the calling NXE process
     -tg|--target: the target server to use
                   (default: $_target )
     -b|--baseport: port used to contact the server
     -p1|--preset-1: Preset configuration 1
                     (default: $_p1 ) 
     -p2|--preset-2: Preset configuration 2
                     (default: $_p2 )
     -p3|--preset-3: Preset configuration 3
                     (default: $_p3 )  
     -g|--gnet: indicates that the vlan for the gnet should be configured
                (default: $_gnet ) 
     -l|--limit: indicates a strict bandwith limit
                 (default: $_limit )
     -m|--mode: type of bandwidth limitation performed
                 (default: $_mode )

EOF


echo "KO!"
exit 0

}

#Setting default options
_pid=0
_date=0
_timeout=4200
_timeout_ITG=4200
_p1=1
_p2=0
_p3=0
_target="localhost"
_baseport=8999
_gnet=0
_limit=""
_mode=""


_path=`pwd`"/bin"

#Handling the options
while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	--pid)
	    shift
	    _pid=$1
	    shift
	;;
	-d|--date)
	    shift
	    _date=$1
	    shift
	;;
	-t|--timeout)
	    shift
	    _timeout_ITG=`expr $1 * 1000`
	    _timeout=$1
	    shift
	;;
	-b|--baseport)
	    shift
	    _baseport=$1
	    shift
	;;
	-tg|--target)
	    shift
	    _target=$1
	    shift
	;;
	-p1|--preset-1)
	    shift
	    _p1=1
	    _p2=0
	    _p3=0
	;;
	-p2|--preset-2)
	    shift
	    _p1=0
	    _p2=1
	    _p3=0
	;;
	-p3|--preset-3)
	    shift
	    _p1=0
	    _p2=0
	    _p3=1
	;;
	-g|--gnet)
	    shift
	    _gnet=1
	;;
	-l|--limit)
	    shift
	    _limit=$1
	    shift
	;;
	-m|--mode)
	    shift
	    _mode=`echo $1|tr '[:lower:]' '[:upper:]'`
	    shift
	;;
	--path)
	    shift
	    _path=$1
	    shift
	;;
	*)
	    echo "$1: Unknown option"
	    help
	;;
    esac
done

if [ "$_gnet" -eq "1" ]; then
    sudo $_path/config_vlan.sh > /dev/null
fi


if [ -n "$_mode" ]; then
    sudo $_path/limit_bandwidth.sh --target $_target --limit $_limit --mode $_mode
fi


log_file="/tmp/log_bckground_"`hostname`"_"$_pid"_"$_date"_"$_p1$_p2$_p3".log"


options="-x $log_file -a $_target -t $_timeout_ITG -rp $_baseport"


if [ "$_p1" -eq "1" ]; then
    #Setting extra option for the preset 1 that is ??
    options=$options" -v 1.1 130000 -E 0.2"	   
elif [ "$_p2" -eq "1" ]; then
    #Setting extra option for the preset 2 that is ??
    options=$options
elif [ "$_p3" -eq "1" ]; then
    #Setting extra option for the preset 3 that is ??
    options=$options
fi


ITGSend $options > /dev/null &



sleep $_timeout
kill -9 $!
kill -9 $!

echo "OK!"

if [ -n $_mode ]; then
    sudo $_path/limit_bandwidth.sh --reset --target $_target --mode $_mode
fi

#First the switches
if [ "$_gnet" -eq "1" ]; then
	    sudo $_path/config_vlan.sh --reset > /dev/null
fi


exit 0
