#!/bin/bash
#A simple script to launch a client (basically a iperf client) a given number 
#of times using a list of interwait times and a list of volumes to transfer 
#
#Once again, as it is difficult to set up, the parallel thingies are disabled
#
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: launch_client_law.sh,v 1.5 2008-11-06 11:26:40 rguillie Exp $


help()
{
cat <<EOF

A script to launch a client for the NXE
Usage: $0 [option value]

 
   -h|--help: display this and exit
   --pid:  the pid of the calling NXE process
   -d|--date: the date at which the process is supposed to start
                 (default: $_date ) 
   -t|--timeout: the max time before the script commits suicide
                 (default: $_timeout )
   -b|--baseport: the base port to use for the server
                 (default: $_baseport)
   -v|--volume : indicates the volume of data to send
                 (default: $_volume )
   -c|--continuous : indicates that are sending continously till the timeout occurs
                 (default: $_continuous) 
   -p|--parallel : indicates the number of parallel streams to send the data
                 (default: $_parallel)
   -w|--window : the size of the TCP buffer to use
                 (default: $_buffer_size)
   -u|--udp :    using UDP
                 (default: $_udp)  
   -bd|--bandwidth: (UDP only)  
                 (default: $_bandwidth )
   -tg|--target: the target server to use
                 (default: $_target )
   -pr|--protocol: the name of the TCP protocol to use
                 (default: $_protocol )     
   -l|--limit: indicates a strict bandwith limit (ethtool thingie)
                 (default: $_limit )
   -m|--mode: indicates the type of bandwidth limitation performed
                 (default: $_mode )
   -g|--gnet: indicates that the vlan for the gnet should be configured
              (default: $_gnet )
   -wb|--web100: retrieving the web100 logs
              (default: $_web100 )
   -1g|--1g_bottleneck: configuring the 1G bottleneck, though it is not a good idea to do it at this stage as it is 
                        takes about 40s for it to be properly set up (dhcp discovery takes a long time) 
              (default: $_1G_bottleneck )

EOF

echo "KO!"

exit 0
}

#Setting default options

_timeout=700
_volumeunit="30000M"
_baseport=5001
_parallel=1
_continuous=0
_buffer_size="4M"
_udp=0
_pid=0
_date=0
_target="localhost"
_protocol="reno"

_limit=""
_mode="TBF"
_size="3000M"
_interval="0"
_gnet=0
_web100=""
_1G_bottleneck=0
_path=`pwd`"/bin"
index=0

#Handling the options

while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	--pid)
	    shift
	    _pid=$1
	    shift
	;;
	-d|--date)
	    shift
	    _date=$1
	    shift
	;;
	-t|--timeout)
	    shift
	    _timeout=$1
	    shift
	;;
	-b|--baseport)
	    shift
	    _baseport=$1
	    shift
	;;
	-v|--volume)
	    shift
	    _volume=$1
	    shift
	;;
	-c|--continuous)
	    shift
	    _continuous=1
	;;
	-p|--parallel)
	    shift
	    _parallel=$1
	    shift
        ;;
	-w|--window)
	    shift
	    _buffer_size=$1
	    shift
        ;;
	-u|--udp)
	    shift
	    _udp=1
	;;
	-bd|--bandwidth)
	    shift
	    _bandwidth=$1
	    shift
	;;
	-tg|--target)
	    shift
	    _target=$1
	    shift
	;;
	-pr|--protocol)
	    shift
	    _protocol=$1
	    shift
        ;;
	-l|--limit)
	    shift
	    _limit=$1
	    shift
	;;
	-i|--interval)
	    shift
	    _interval=$1
	    shift
	;;
	-s|--size)
	    shift
	    _size=$1
	    shift
	;;
	-g|--gnet)
	    shift
	    _gnet=1
	;;
	-m|--mode)
	    shift
	    _mode=`echo $1 | tr '[:lower:]' '[:upper:]'`
	    shift
	;;
	-wb|--web100)
	    shift
	    _web100=`echo $1| sed 's/:/ /g'`
	    shift
	;;
	-1g|--1g-bottleneck)
	    shift
	    _1G_bottleneck=1
	;;
	*)
	    echo "$1: Unknown option"
	    help
	;;
    esac
done


#Changing the protocol
sudo $_path/change_congestion_control.sh $_protocol > /dev/null


#First the switches
if [ "$_gnet" -eq "1" ]; then
    sudo $_path/config_vlan.sh > /dev/null
fi


if [ -z "$_limit" ]; then
    #Resetting the interface to the maximum value, just in case
#    echo pouet > /dev/null
    sudo $_path/limit_bandwidth.sh --target $_target --reset --mode $_mode
else
    #Setting the new limit
    sudo $_path/limit_bandwidth.sh --target $_target --limit $_limit --mode $_mode
#    echo pouet > /dev/null

fi

if [ "$_1G_bottleneck" -eq "1" ]; then
    sudo $_path/config_1g_bottleneck.sh
fi
    
if [ ! -z $_web100 ]; then
  next_web100=`$_path/get_next_connexion_id.sh`
  outfile_web100="/tmp/outfile_"`hostname`"_"$_target"_"$_pid"_"$_date"_web100_" 
fi




options="-c $_target" 

if [ "$_continuous" -eq "1" ]; then
    options=$options" -t $_timeout"
fi
#else

if [ "$_udp" -eq "1" ]; then
   options=$options" -u -b $_bandwidth"

fi


#Remains of the global options
if [ "$_mode" = "TCP" ]; then
    #we need to limit the 
    #currently, we can only do it using the buffer size
    rtt=`ping -c 3 $_target | tail -n 1 | cut -d'/' -f 5`
    buffer_size=`echo "scale = 0; $rtt * $_limit * 1000 /8" |bc`"K"

    options=$options" -w "$buffer_size
    
else
    options=$options" -w "$_buffer_size
fi

index=1
repeat=`echo $_size | sed 's/:/ /g' | wc -w`

while [ $index -le $repeat ]; do

    volumeunit=`echo $_size | cut -d':' -f $index`
    sleeptime=`echo $_interval  | cut -d':' -f $index`

    #Separing unit and volume 
    #unit=`echo $volumeunit | sed 's/[0-9]*//' `
    #volume=`echo $volumeunit | sed 's/[M,K]$//'`

    #Splitting the volume between the parallel streams and adding back the unit
    #volume=`expr $volume / $_parallel`
    #_volumeunit=$volume$unit
    
    locoptions=$options" -n "$volumeunit

    #fi

    if [ -z $sleeptime ]; then
	sleeptime=0
    fi
    if [ -z $volumeunit ]; then
	volumeunit="15M"
    fi


#echo $options

#base_port=`expr $_baseport + $_date`
currentport=$_baseport

# index=1

# while [ $index -le $_parallel ]; do

     #wait for the intertransfer interval
     sleep $sleeptime
     if [ ! -z $_web100 ]; then
	 #the parameters shouldn't be fixed
	 
	 sudo readvartimer $next_web100 -t 3600 -f 10000000 $_web100 > $outfile_web100""$index".log" &
	 pid_web100=$!
     fi

     iperf $locoptions -p $currentport > "/tmp/plouf"$index 2> "/tmp/stderr"$index  
#    currentport=`expr $currentport + 1`
#    index=`expr $index + 1`
    #keeping a string with all the pids forked
#    lpid=$lpid" "$!
    
     #\o/ iperf don't know anything about return code convention
     if [ -n "`cat /tmp/stderr$index | grep \"Connection refused\"`" ]; then
	 echo "OK"!
	 sudo $_path/limit_bandwidth.sh --target $_target --reset --mode $_mode
	 #First the switches
	 if [ "$_gnet" -eq "1" ]; then
		 sudo $_path/config_vlan.sh --reset > /dev/null
  	 fi
 	 exit 1
     fi

    index=`expr $index + 1`
    
    if [ ! -z $_web100 ]; then
	kill -9 $pid_web100 > /dev/null
	next_web100=`expr $next_web100 + 1`
    fi
   

done

#sleeping for the timeout
#sleep $_timeout
#killing all the pids forked
#kill -9 $lpid > /dev/null
#kill -9 $lpid > /dev/null

echo "OK!"

#Resetting the limitation
sudo $_path/limit_bandwidth.sh --target $_target --reset --mode $_mode



#First the switches
if [ "$_gnet" -eq "1" ]; then
	    sudo $_path/config_vlan.sh --reset > /dev/null
fi

if [ "$_1G_bottleneck" -eq "1" ]; then
    sudo $_path/config_1g_bottleneck.sh --reset > /dev/null
fi


exit 1 
