#!/bin/bash
#A script to launch a gridftp server (basically a iperf server)
#
#
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: launch_gridftp_server.sh,v 1.2 2008-11-06 11:26:41 rguillie Exp $


help()
{
cat <<EOF

A script to launch a server for the NXE
Usage: $0 [option value]

   -h|--help: display this and exit
   --pid:  the pid of the calling NXE process
   -d|--date: the date at which the process is supposed to start
                 (default: $_date ) 
   -t|--timeout: the max time before the script commits suicide
                 (default: $_timeout )
   -b|--baseport: the base port to use for the server 
                 (default: $_baseport)
   -p|--parallel : indicates the number of parallel streams to send the data
                 (default: $_parallel)
   -w|--window : the size of the TCP buffer to use
                 (default: $_buffer_size)
   -u|--udt :    using UDT
                 (default: $_udt)
   -g|--gnet : setting the vlan for the gnet
               (default: $_gnet)
   -1g|--1g-bottleneck: setting the 1g bottleneck but it is probably not a good idea because it takes about 40s before
                        starting (dhcp reponse is slow)   
  
EOF

echo "KO!"
exit 0
}

#Setting default options

_timeout=700
_baseport=5000
#_parallel=1
#_continuous=0
_buffer_size="4M"
_udt=0
_pid=0
_date=0
_protocol="reno"
_gnet=0 
_1g_bottleneck=0
globus_home="/usr/local/globus-4.2.0/"

_path=`pwd`"/bin"
#Handling the options

while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	--pid)
	    shift
	    _pid=$1
	    shift
	;;
	-d|--date)
	    shift
	    _date=$1
	    shift
	;;
	-t|--timeout)
	    shift
	    _timeout=$1
	    shift
	;;
	-b|--baseport)
	    shift
	    _baseport=$1
	    shift
	;;	
	-p|--parallel)
	    shift
	    _parallel=$1
	    shift
        ;;
	-w|--window)
	    shift
	    _buffer_size=$1
	    shift
        ;;
	-u|--udt)
	    shift
	    _udt=1
	;;
	-tg|--target)
	    shift
	    #We don't care about target, we are a server
	    shift
	;;
	-pr|--protocol)
	    shift
	    _protocol=$1
	    shift
	;;
	-g|--gnet)
	    shift
	    _gnet=1
	;;
	-1g|--1g-bottleneck)
	    shift
	    _1g_bottleneck=1
	;;
	--path)
	    shift
	    _path=$1
	    shift
	;;
	*)
	    echo "$1: Unknown option"
	    help
	;;
    esac
done

sudo $_path/change_congestion_control.sh $_protocol > /dev/null


#rajouter pid et date dedans
logfile="/tmp/log_gridftp_"`hostname`"_"$_pid"_"$_date"_"`cat /proc/sys/net/ipv4/tcp_congestion_control`"_"$_buffer_size

#We just need to generate the good command line for the server iperf

options="-aa -anonymous-user anonymous -dc-whitelist udt,tcp,gsi -log-module stdio_ng -log-level info,warn,error"


#First the switches
if [ "$_gnet" -eq "1" ]; then
    sudo $_path/config_vlan.sh
fi

if [ "$_1g_bottleneck" -eq "1" ]; then
    sudo $_path/config_1g_bottleneck.sh
fi


if [ "$_udt" -eq "1" ]; then
    options=$options" -u"
    logfile=$logfile"_UDT"
else
    logfile=$logfile"_TCP"
fi

options=$options" -logfile $logfile"


#Common part
#options=$options" -w "$_buffer_size 

options=$options" -p $_baseport"

#base_port=`expr $_baseport + $_date`
$globus_home/sbin/globus-gridftp-server $options & 
    
    #iperf $options -p $currentport > $logfile"_"$currentport"_flow"$index"_"$_parallel".log" 2> /dev/null & 
    #currentport=`expr $currentport + 1`
    #index=`expr $index + 1`
    #Keeping a string with all the pids forked 
    lpid=$! 

#done


#sleeping for the timeout and exiting 
sleep $_timeout
#Killing all the pids forked  
kill -9 $lpid
kill -9 $lpid


#First the switches
if [ "$_gnet" -eq "1" ]; then
	    sudo $_path/config_vlan.sh --reset > /dev/null
fi

if [ "$_1g_bottleneck" -eq "1" ]; then
    sudo $_path/config_1g_bottleneck.sh --reset > /dev/null
fi

echo "OK!"
exit 1
