#!/bin/bash
#A simple script using ethtool to strictly limit the bandwidth achievable
#Could be done using queue-disc also, but for the moment, the resolution of
#1000Mbps/100Mbps/10Mbps seems to be enough
# 
#Addition of the handling of some common qdisc, beware that HTB and CBQ don't seem
#to behave properly 
#Should be run as root
# 
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: limit_bandwidth.sh,v 1.2 2008-11-06 11:26:41 rguillie Exp $




help()
{
cat <<EOF

A script to strictly limit the bandwidth achievable  
Usage: $0 [option value]

  -h|--help: display this and exit
  -r|--reset: resetting the limit to the 1000Mbps value 
              (default: $_reset )
  -lt|--limit: the value of the limit to set
              (default: $_limit )
  -tg|--target: the target we need to consider to find the correct network interface 
  -m|--mode: defines the way the bandwidth limitation is made
             (default: $_mode )   

EOF

exit -1 

}

#Setting the options
_limit=100
_reset="N"
_target=`hostname`
_targetIP="127.0.0.1"
_mode="TBF"

while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	-l|--limit)
	    shift
	    _limit=$1
	    shift
	;;
	-r|--reset)
	    shift
	    _reset="Y"
	;;
	-tg|--target)
	    shift
	    _target=$1
	    shift
	;;
	-m|--mode)
	    shift
	    _mode=$1
	    shift
	;;
	*)
	    echo "$1: Unknown option"
	    help
	;;	
    esac
done

#First, we need to find the IP of the target

isIP=`echo $_target| grep -E --regexp="^([0-9]{1,3}\.){3}[0-9]{1,3}"`
#echo $isIP
if [ -z $isIP ]; then
    _targetIP=`host $_target|sed 's/\t/:/g' | cut -d':' -f3`  
#    echo $_targetIP
else
    _targetIP=$_target
fi

#Then we find the interface name
interface=`ip route get $_targetIP | head -n 1 | sed 's/dev /:/' | sed 's/  src/:/' | cut -d':' -f2` 

#echo $interface
ethtool -K $interface tso off


if [ "$_mode" = "PHY" ]; then

    #using ethtool to enforce the policy
    if [ $_reset = "Y" ]; then
	ethtool -s $interface autoneg off speed 1000
    else
	ethtool -s $interface autoneg off speed $_limit
    fi

elif [ "$_mode" = "TBF" ]; then

  if [ $_reset = "Y" ]; then
      tc qdisc del dev $interface root
  else
      burst=`expr $_limit \* 4`kb

      tc qdisc add dev $interface root tbf rate $_limit"mbit" burst $burst latency 100ms
  fi


elif [ "$_mode" = "HTB" ]; then

      if [ $_reset = "Y" ]; then
	  tc qdisc del dev $interface root
      else
	  tc qdisc add dev $interface root handle 1: htb default 1
	  tc class add dev $interface parent 1: classid 1:1 htb rate $_limit"mbit" ceil $_limit"mbit"  
      fi 

elif [ "$_mode" = "CBQ" ]; then

      if [ $_reset = "Y" ]; then
	  tc qdisc del dev $interface root
      else
	  tc qdisc add dev $interface root handle 1: cbq bandwidth 1000mbit avpkt 1514
	  tc class add dev $interface parent 1: classid 1:1 cbq bandwidth 1000mbit rate $_limit"mbit" weight $_limit avpkt 1514 bounded isolated
	  tc filter add dev $interface parent 1: protocol ip prio 0 u32 match u32 00000000 00000000 at 0 flowid 1:1

      fi

elif [ "$_mode" = "TCP" ]; then

    #We are doing nothing here but it could be cool to modify the kernel to
    #have a sysctl call to the tcp_clamp thingie
    if [ $_reset = "Y" ]; then
	echo "pouet" > /dev/null
    else
	echo "pouet" > /dev/null
    fi

fi


