#!/bin/bash
#A script to transform the logs retrieved from the test to have something useable to draw
# with gnuplot
#
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: preparing_logs.sh,v 1.2 2008-11-06 11:26:41 rguillie Exp $


help()
{
cat <<EOF
A script to transform the logs generated by iperf to something useable with gnuplot 
Usage: $0 [option value]

       -h|--help: display this and exit
       -f|--file: file containing the names of the logs files to modify
                  (default: $_files_log)
       -s|--sleep: file containing the time spent in sleep for the given log
                  (default: $_sleep_file) 
       -w|--write: the name of the file used to output the result
                  (default: $_output_file) 
       -o|--offset: indicate the offset between start-time of iperf server and iperf client
		 (default: $_offset)
EOF

exit 0

}

#Default options
_files_log="./log_list"
_sleep_file="./sleep_list"
_output_file="./log_formated_for_gnuplot"
_offset=2

#Handling options
while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    help
	;;
	-f|--file)
	    shift
	    _files_log=$1
	    shift
	;;
	-s|--sleep)
	    shift
	    _sleep_file=$1
	    shift
	;;
	-w|--write)
	    shift
	    _output_file=$1
	    touch $_output_file
	    shift
	;;
	-o|--offset)
	    shift
	    _offset=$1
	    shift
	;;
	*)
	    echo "$1 : Unknown command"  
	    help
    esac

done 

index=1
time_slept=0
max_size=0

rm -f  ./ticks
rm -f  ./transfer_time

#Looping 
for file in `cat $_files_log`; do

echo `date`" : treating $file "

    if [ "$index" -eq "1" ]; then
	next_sleep=`cat $_sleep_file | head -n $index`
    else
	index_1=`expr $index - 1`
	next_sleep=`cat $_sleep_file | head -n $index | tail -n 1 | cut -f5 -d':'`
	#Adding $_offset as there is a $_offset seconds' sleep between the launch of the 
	#clients and our reference is the start of the first node
        next_sleep=`expr $next_sleep + $_offset`
    fi

    #echo "$index , $next_sleep" 	

    #index=`expr $index + 1`
    rm -f /tmp/tmp_shifting.$$
    
    time_slept=$next_sleep

    #_2xtime_slept=`expr 2 \* $time_slept`
    #count=0
    
    

    echo $time_slept >> ./ticks

#Filling the blanks to avoid problems while merging files
#    while [ $count -lt $_2xtime_slept ]; 
#      do
  #echo "$count : $_2xtime_slept"

#      if [ `expr $count % 2` -eq 0 ]; then
#	  echo `expr $count / 2`" 0" >> /tmp/tmp_shifting.$$ 
#      else
#	  echo `expr $count / 2`".5 0" >> /tmp/tmp_shifting.$$
#      fi
#      
#      count=`expr $count + 1`
#    done

#Padding the beginning of the files
    seq -f"%f 0" 0 .5 $time_slept  | head --lines=-1 > /tmp/tmp_shifting.$$
    
    #cat /tmp/tmp_shifting.$$ | head --lines=-1 > /tmp/tmp.$$ 

    
    #echo "----"
    #echo $time_slept
    #seq -f"%f 0" 0 .5 $time_slept  | head --lines=-1
    #echo "------"
    #cat /tmp/tmp_shifting.$$
    #echo "----"

#Removing the last line (average on the whole period) and the 5 first (header)
#Cleaning the file by fetching only the tick and the bandwith measure
#Shifting the rest of the file with awk 
#cat $file | sed 's/-/ /g' | sed 's/ \{2,\}/ /g' | cut -d' ' -f 3,8 | awk '{print $1+$time_slept" "$2;}' >> /tmp/tmp_shifting.$$

    #Keeping the last line to save the average and the total transfer time 
    #$4 is the transfer time, $6 the amount of data transfered (just in case) and $8 the average goodput 
    cat $file | grep -v "local" | tail -n 1 | sed 's/-/ /' | awk '{printf "%f %f %f %f\n",$4,$8,$6,$4+'$time_slept';}' >> ./transfer_time

    size_file=`cat $file | wc -l`
    #size_file=`expr $size_file - 2`
    #size_without_header=`expr $size_file - 4`
    
     #If there is a second connection at the end, we have to adjust again
     #the size of the file
     nb_connection=`cat $file | grep "local" | wc -l`
     #echo $nb_connection
     #if [ "$nb_connection" -ge "2" ]; then
     size_file=`expr $size_file - $nb_connection - 1`
     size_without_header=`expr $size_file - 4`
     #fi

    #echo "File size: $size_file"
    #echo "Size without header: $size_without_header "

#    echo " cat $file | grep -v "local" | head -n $size_file |tail -n $size_without_header| sed 's/-/ /g' | sed 's/ \{2,\}/ /g' | grep -v "received" | cut -d' ' -f 3,8 | awk '{printf "%f %f\n",$1+'$time_slept',$2;}'"

    cat $file | grep -v "local" | head -n $size_file |tail -n $size_without_header| sed 's/-/ /g' | sed 's/ \{2,\}/ /g' | grep -v "received" | cut -d' ' -f 3,8 | awk '{printf "%f %f\n",$1+'$time_slept',$2;}' >> /tmp/tmp_shifting.$$
	
    cp /tmp/tmp_shifting.$$ ./col$index
    #echo $file " is " col$index 
    index=`expr $index + 1`


    echo `expr \( $size_without_header \/ 2 \) + $time_slept` >> /tmp/tmp_size.$$

    #cat /tmp/tmp_size.$$

#Pasting with the rest 
    #paste /tmp/tmp_shifting.$$ $_output_file > /tmp/tmp_shifting_concat.$$
    #cat /tmp/tmp_shifting_concat.$$ > $_output_file



done

#max_size=`wc -l ./col* | sort -n | tail -n 2 | head -n 1 | awk '{printf "%i", $1;}'`
stop=`cat /tmp/tmp_size.$$ | sort -n | tail -n 1`
max_size=`expr $stop \* 2`
#stop=`expr $max_size \/ 2` 



#if [ `expr $max_size % 2` -eq 1 ]; then
#   stop=$stop".5"
#fi

#echo "Max_size= $max_size : stop = $stop "

count=1
while [ $count -lt $index ]; do 

    current_size=`wc -l ./col$count | cut -d' ' -f1` 
    #echo "treating file ./col$count with $current_size"

    if [ $current_size -lt $max_size ]; then
        
        start=`expr $current_size \/ 2`
        if [ `expr $current_size % 2` -eq 1 ]; then
        #  start=`expr $start + 1`  
        #else
          start=$start".5"
        fi
   
        #echo "Padding end of file ./col$count from $start to $stop"
        seq -f"%f 0" $start 0.5 $stop >> ./col$count      
    
    else
        echo "$stop 0" >> ./col$count
    fi 
    

    paste ./col$count $_output_file > /tmp/tmp_shifting_concat.$$
    cat /tmp/tmp_shifting_concat.$$ > $_output_file

    count=`expr $count + 1`
done


#Other things to be done
generate_time.sh > ./time

cat /tmp/tmp_size.$$ >> ./ticks


rm /tmp/tmp_shifting*.$$
rm /tmp/tmp_size.$$
