#!/bin/bash
#A simple script to handle a reservation on G5K using oar
#Returns a list of nodes that have been reserved on G5K
#
#For cleaning purposes, the list/reservation number are 
#stored in a file on a local /tmp/ file
#
#Should be OAR 2.0 compliant
#
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: resa_g5k.sh,v 1.4 2008-11-06 11:26:41 rguillie Exp $


help()
{
cat <<EOF
Usage $0 [option value]

   -h|--help: display this and exit
   -n|--nodes: number of nodes used during the test (on one side)
               (default: $_number_test_nodes)
   -w|--walltime: max time for which the nodes wil be reserved
                 (default: $_walltime)
   -q|--queue: the name of the queue used
                 (default: $_queue)
   -t|--type: the type of the reservation
                 (default: $_type)
   -c|--cluster: the name of the cluster used (thank you oar2.0)
                 (default: $_cluster)
   -s|--site: provide the name of the cluster on which to reserve
                 (default: $_site) 
   -r|--reservation-number: indicates that the reservation number has already been made (in case we use the pid to get it)
                 (default: $_reservation_number)
   -p|--pid: the pid of the programm calling this script to allow multiple simultaneous usage   
   -d|--delete: delete the reservation corresponding to the reservation number/pid
                (default: $_delete)
   -rt|--retries: the number of seconds/retries before letting go a reservation
   		(default: $_retries)
   -dt|--date: indicate a date from which we need to perform the reservation
   		(default: $_date)
   -sl|--sleep: indicate the time that we will wait for before trying to look at the file supposed to contain the reservation nodes
                (just a magic trick to handle the advance reservation, because date calculation in shell is complicated,
                 could do the same with retries, but it is useless to poll the file every second as we know when to wake up)  
                (default: $_sleep)
   -1g|--1G_bottleneck: indicates that we will be using the 1G bottleneck through a contention switch, that means that we need to
                        fill the second column with the adaptaion of the name with the string provided

                (default: $_1g)  
   -pr|--properties: extra properties to ask for during the reservation
                (default: $_properties )


EOF

exit 0

}
   
#Setting default options
_number_test_nodes=1
_walltime=1
_site=""
_queue="default"
_type="deploy"
_reservation_number=0
_pid=""
_delete=0
_cluster=""
_retries=150
_date=`date +'%Y-%m-%d %H:%M:%S'`
_sleep=0
_1g=""
_properties=""

#Handling the options
while [ $# -gt 0 ]; do

    case $1 in
        -h|--help)
            help
        ;;
	-n|--nodes)
	    shift
	    _number_test_nodes=$1
	    shift
	;;
	-w|--walltime)
	    shift
	    _walltime=$1
	    shift
	;;
	-q|--queue)
	    shift
	    _queue=$1
	    shift
	;;
	-s|--site)
	    shift
	    _site=$1
	    shift
	;;
	-r|--reservation-number)
	    shift
	    _reservation_number=1
	    _pid=$1
	    shift
        ;;
	-p|--pid)
	    shift
	    _pid=$1
	    shift
	;;
	-d|--delete)
	    shift
	    _delete=1
	    _pid=$1
	    shift
	;;
	-c|--cluster)
	    shift
	    _cluster=$1
	    shift
	;;
	-t|--type)
	    shift
	    _type=$1
	    shift
	;;
	-dt|--date)
	    shift
	    _date=$1
	    shift
	;;
	-sl|--sleep)
	    shift
	    _sleep=$1
	    shift
	;;
	-1g|--1G_bottleneck)
	    shift
	    _1g=$1
	    shift
	;;
	-pr|--properties)
	    shift
	    _properties=$1
	    shift
	;;
        *)
	    echo "Unknown option"
	    help
    esac
done

if [  $_reservation_number -eq 1 ]; then 
    #Okay, so the reservation should have already be done
    
    if [ "$_pid" = "" ]; then
	exit -1
    else
	job_id=`cat "/tmp/NXE_id_job_"$_pid"_"$_cluster`
	cat "/tmp/OAR_"$job_id"_uniq_"$_pid"_"$_cluster
	exit 0
    fi
fi


	    
if [ $_delete -eq 1 ]; then
    #We need to clean this reservation
    
    if [ "$_pid" = "" ]; then
	exit -1
    else
	job_id_file="/tmp/NXE_id_job_$_pid""_$_cluster"
	oardel `cat $job_id_file`
	rm /tmp/*$_pid
    fi

else

#Should need to do something about the site 
    
    #Because oar scripts are completly fucked up
    #echo "#!/bin/bash \n /bin/sleep 3600000 &" > ./script_idiot
    #chmod +x ./script_idiot
    
    #options="-t $_queue -l walltime=$_walltime -l nodes=$_number_test_nodes -p cluster='$_cluster' -r '`date +'%Y-%m-%d %H:%M:%S'`' "
   
    job_id_file="/tmp/NXE_id_job_$_pid""_$_cluster"
  
    #echo $options
    #echo $job_id_file

    #options="-q $_queue -t $_type -l nodes=$_number_test_nodes,walltime=$_walltime -p \"cluster='$_cluster'\" -r \"$_date\" -n NXE_$_pid"

    if [ ! -z $_properties ]; then
	#options=$option" -p "$_properties 
	properties=" and "$_properties
    fi


    oarsub -q $_queue -t $_type -l nodes=$_number_test_nodes,walltime=$_walltime -p "cluster='$_cluster' and state='Alive' $properties" -r "$_date" -n "NXE_"$_pid > $job_id_file
    #oarsub $options > $job_id_file

    job_id=`cat $job_id_file | tail -n 3 | head -n 1 | cut -d'=' -f2`

    if [ $_sleep -gt 0 ]; then
	sleep $_sleep
    fi
	
    #Rewriting directly the job_id in the file to avoid confusion, if it is re-read by anyone else 
    echo $job_id > $job_id_file 

    file_node="/var/lib/oar/"$job_id
    file_node_uniq="OAR_"$job_id"_uniq_"$_pid"_"$_cluster
#Here we must do something if we want to add a new column to handle various interfaces 
    #echo $file_node_uniq
    
    count=0
    
    #Saving the uniq list of nodes in a file with the pid to be able to retrieve it later on
    while [  ! -e $file_node ]; do
	sleep 1
	count=`expr $count + 1`
    	if [ $count -gt $_retries ]; then		
		exit -1
    	fi	

    done

    cat $file_node | uniq > /tmp/$file_node_uniq
    
    if [ -z $_1g ]; then
	cat /tmp/$file_node_uniq
    else
	file_node_uniq_1g="/tmp/"$file_node_uniq"_1g"
	for line in `cat /tmp/$file_node_uniq`; do
	    echo $line `echo "$line" | sed "s/\./$_1g\./"` >> $file_node_uniq_1g
	done
	mv $file_node_uniq_1g /tmp/$file_node_uniq
	cat /tmp/$file_node_uniq
    fi
	

fi

#Cleaning stuff
#rm -f ./script_idiot


exit 0
