#!/bin/bash
#A script to wrap the preparation of the logs and to generate automatically 
#gnuplot command to display it (even to save it if we really care)
#Assumes it is being run in the directory to where the logs were retrieved
#
#
# Copyright or Copr. INRIA/RESO - Romaric GUILLIER, Pascale VICAT-BLANC PRIMET 
#
# Pascale VICAT-BLANC PRIMET - <pascale.primet@inria.fr>
# Romaric GUILLIER           - <romaric.guillier@ens-lyon.fr>
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# 
# For more information, see the file COPYING
#
# $Id: wrapping_logs_preparation.sh,v 1.2 2008-11-06 11:26:41 rguillie Exp $

help()
{
cat <<EOF
A script to prepare the logs after they have been generated and retrieved 
somewhere

Usage $0 [Option value]

    -h|--help: display this and exit
    -n|--number_nodes: indicates the number of nodes involved in the test
                       (default: $_number_nodes)
    -fp|--file_protocol: indicates the file containing the name of the protocol
                       (default: $_file_protocol)
    -fs|--file_sleep: indicates the time slept by each nodes
                      (default: $_file_sleep)
    -bn|--base_name: indicates the base name for the output log
                     (default: $_base_name)
    -in|--image_name: indicates the base name for the image output
                     (default: $_image_base_name)
    -ie|--image_extension: indicates the image extension to be used by gnuplot for output
                     (default: $_image_extension)
    -if|--image_format: indicates the 'set terminal' option to be used by gnuplot for output
                     (default: $_image_format)
    -o|--offset: indicates the offset between start-time of a iperf server and an iperf client
		     (default: $_offset)
    -sA|--siteA: indicates the name of the server side of the test
                 (default: $_siteA)
    -sB|--siteB: indicates the name of the client side of the test
                 (default: $_siteB)
EOF

exit 0

}


#default values 
_file_protocol="./protocol"
_file_sleep="./syslog_runtest.log"
_number_nodes=2
_base_name="log_formated"

_image_base_name="throughput_mesure"
_siteA="nancy"
_siteB="rennes"
_image_extension="eps"
_image_format="postscript landscape color solid \"Helvetica-Bold\" 16 lw 2.5"
#_image_format="postscript landscape"

_offset=2

#handling the options

while [ $# -gt 0 ]; do

    case $1 in
	-h|--help)
	    shift
	    help
	;;
	-n|--number_nodes)
	    shift
	    _number_nodes=$1
	    shift 	
	;;
	-fs|--file_sleep)
	    shift
	    _file_sleep=$1
	    shift
	;;
	-fp|--file_protocol)
	    shift
	    _file_protocol=$1
	    shift
	;;
	-bn|--base_name)
	    shift
	    _base_name=$1
	    shift
	;;
	-sA|--siteA)
	    shift
	    _siteA=$1
	    shift
	;;
	-sB|--siteB)
	    shift
	    _siteB=$1
	    shift
	;;
	-ie|--image_extension)
	    shift
	    _image_extension=$1
	    shift
	;;
	-if|--image_format)
	    shift
	    _image_format=$1
	    shift
	;;
	-o|--offset)
	    shift
	    _offset=$1
	    shift
	;;
	-in|--image_name)
	    shift
	    _image_base_name=$1
	    shift
	;;
	*)
	    echo "Unknown option: $1"
	    exit -1
    esac

done

if [ `ls $_base_name* | wc -l` -gt 0 ] ; then
    echo "Warning: There are already files called $_base_name something. Continue ?" 
    read
fi


index=1
#Switching on the protocols
for protocol in `cat $_file_protocol`;
do

  gnuplot_file=$_base_name"_$protocol"

  #
 
  #echo "cat $_file_sleep | cut -d':' -f5 | sed 's/ //g' | head -n `expr $index \* $_number_nodes` | tail -n $_number_nodes > /tmp/sleep_list.$$"
 
  cat $_file_sleep | cut -d':' -f5 | sed 's/ //g' | head -n `expr $index \* $_number_nodes` | tail -n $_number_nodes > /tmp/sleep_list.$$

  #Selecting the correct log files  
  ls --color=none *_$protocol\_*.log | sort > /tmp/log_list.$$
  
  if [ `cat /tmp/log_list.$$ | wc -l` -ne $_number_nodes ]; then
      echo "Warning: Log files' number not consistent with the number of nodes: $_number_nodes. Continue ?"  
      read
  fi

  preparing_logs.sh -s /tmp/sleep_list.$$ -f /tmp/log_list.$$ -w $gnuplot_file -o $_offset

  #Generating the GNUplot command
  i=1
  gnuplot_command=""
  sum=""
  while [ $i -le $_number_nodes ] ; do

      _2i=`expr 2 \* $i`
      _2i_1=`expr $_2i - 1`
            
      gnuplot_command=$gnuplot_command",   '$gnuplot_file' u $_2i_1:$_2i w l notitle " #\"$protocol server $i\""   
      
      if [ "$sum" != "" ]; then
	  sum=$sum"+\$$_2i"
      else
	  sum="\$$_2i"
      fi
      i=`expr $i + 1`
  done

  gnuplot_command_individual='set key top left; set yrange [0:1200]; '`echo $gnuplot_command | sed 's/, /plot /' | sed "s/notitle/t \"$_number_nodes Individual $protocol flows\"/"`
  gnuplot_command='set key top left; set yrange [0:1600]; set ytics nomirror ; set y2tics 1000; '`echo $gnuplot_command | sed 's/, /plot /'` 
  gnuplot_command_sum=$gnuplot_command", '$gnuplot_file' u 1:($sum) w l t \"$protocol Sum\" axis x1y2"

  	
  gnuplot_command_aggregate="set key top left; set yrange [0:10000]; plot '$gnuplot_file' u 1:($sum) w l t \"$protocol Aggregate Goodput\" "

  image_name=$_image_base_name"_"$protocol"_"$_siteA"_"$_siteB"."$_image_extension
  image_name_sum=$_image_base_name"_"$protocol"_"$_siteA"_"$_siteB"_sum."$_image_extension	

  image_name_individual=$protocol"_"$_siteA"_"$_siteB"_individual."$_image_extension
  image_name_aggregate=$protocol"_"$_siteA"_"$_siteB"_aggregate."$_image_extension

  echo "*************************************************************************"	
  echo $gnuplot_command	
  echo "*************************************************************************"
  echo $gnuplot_command_sum
  echo "*************************************************************************"
  echo $gnuplot_command > ./gnuplot_command
  echo $gnuplot_command_sum > ./gnuplot_command_sum

  echo $gnuplot_command_individual > ./gnuplot_command_individual
  echo $gnuplot_command_aggregate > ./gnuplot_command_aggregate

  echo "Generating $image_name"
  echo "set xlabel \"T (s)\" ; set ylabel \"Goodput (Mbps)\" ; set terminal $_image_format ; $gnuplot_command" | gnuplot > $image_name
  
  echo "Generating $image_name_sum"
  echo "set xlabel \"T (s)\" ; set ylabel \"Goodput (Mbps)\" ; set terminal $_image_format ; $gnuplot_command_sum" | gnuplot > $image_name_sum

  echo "Generating $image_name_aggregate"
  echo "set xlabel \"T (s)\" ; set ylabel \"Goodput (Mbps)\" ; set terminal $_image_format ; $gnuplot_command_aggregate" | gnuplot > $image_name_aggregate
	
  echo "Generating $image_name_individual"
  echo "set xlabel \"T (s)\" ; set ylabel \"Goodput (Mbps)\" ; set terminal $_image_format ; $gnuplot_command_individual" | gnuplot > $image_name_individual


  #echo 'set terminal png; plot "histo2.txt" with boxes' | gnuplot > histo2.png

  index=`expr $index + 1`
done

rm -rf /tmp/*.$$
